# This sample tests function parameter matching logic.


from typing import Any, Callable, Literal


def func1(a: int, *b: int):
    pass


func1(3)
func1(3, 4)
func1(3, *[1, 2, 3])

# This should generate an error
func1(3, "hello")

# This should generate an error
func1(3, 5, 2, "str")

# This should generate an error
func1("hello", 3)

str_list = ["he", "2", "3"]

# This should generate an error
func1(3, *str_list)


def func2(a: str, **b: int):
    pass


func2("hi")
func2("hi", b=3, c=4, d=5)

str_dict = {"a": "3", "b": "2"}

# This should generate a type error
func2("hi", **str_dict)


# This should generate a type error
func2("hi", 3)

# This should generate a type error
func2("hi", b="hi")


def func4(*args: int):
    pass


def func5(a: int, *args):
    pass


tuple1 = (2, 3)
func4(*tuple1)
func5(*tuple1)

# This should generate an error because a is assigned twice.
func2(a="", a="")

# This should generate an error because c is assigned twice.
func2("", c=4, d=5, c=5)


def func6(param1: int, param2: str):
    pass


def func7(*args: Any, param0: int, param1: int, param2: str):
    func6(*args, param1=param1, param2=param2)

    func6(param0, param2=param2)

    # This should generate two errors because param0 has no match
    # and param2 is missing.
    func6(param0, param1=param1)


def func8(
    y: str,
    z: bool = ...,
) -> None: ...


kwargs1: dict[str, int] = {}
# This should generate an error because int is not compatible with str.
func8(z=False, **kwargs1)


class MyStr(str): ...


kwargs2: dict[MyStr, MyStr] = {}
func8(z=False, **kwargs2)


def func9(
    x: int,
    y: str,
    *,
    a: str = ...,
    b: str,
    c: str,
) -> None: ...


kwargs3: dict[str, str] = {}
func9(0, "", **kwargs3)

args4: list[str] = ["hi"]
func9(0, *args4, **kwargs3)

# This should generate an error.
func9(*args4, **kwargs3)


def func10(x: int): ...


func10(1, *())

# This should generate an error.
func10(1, *(1,))

func10(*(1,))

# This should generate an error.
func10(*(1, 1))

# This should generate an error.
func10(*("",))


def func11(y: tuple[int, ...]):
    func10(1, *y)


def func12(x: int, /, y: str):
    pass


# This should generate an error.
func12(1, **{"z": None})


def func13(*, a: str, b: str, c: int | None = None):
    ...


func_args1: dict[Literal["a", "b", "d"], str] = {
    "a": "a",
    "b": "b",
    "d": "d",
}

func13(**func_args1)

func_args2: dict[Literal["a", "b", "c"], str] = {
    "a": "a",
    "b": "b",
    "c": "c",
}


# This should generate an error.
func13(**func_args2)


def func14(cb1: Callable[..., Any], cb2: Any, x: None):
    cb1(**x)  # This should generate an error
    cb2(**x)  # This should generate an error


def func15(cb1: Callable[..., Any], cb2: Any, a: int, b: None | str):
    print(*a)  # This should generate an error
    print(*b)  # This should generate an error
    cb1(*a)  # This should generate an error
    cb2(*b)  # This should generate an error




# This sample tests that recursive type aliases work well with
# a generic dataclass constructor.

from dataclasses import dataclass
from typing import Union, Generic, TypeVar

A = TypeVar("A")
JSON = Union[str, dict[str, "JSON"]]


@dataclass
class Example(Generic[A]):
    val: A


a: JSON = {"a": "b"}
b: JSON = "a"
c: Example[JSON] = Example(a)
d: Example[JSON] = Example("a")
e: Example[JSON] = Example({})
f: Example[JSON] = Example({"a": "b"})
g: Example[JSON] = Example({"a": {"a": "b"}})



# This sample tests the case where a function type is compared to another
# function type where one contains a positional-only marker and the
# other does not.

from typing import Protocol


class _Writer1(Protocol):
    def write(self, a: str, b: str) -> object:
        pass


class Writer1:
    def write(self, a: str, /, b: str):
        pass


def make_writer1(w: _Writer1):
    pass


# This should generate an error because the source function is positional-only.
make_writer1(Writer1())


class _Writer2(Protocol):
    def write(self, a: str, /, b: str) -> object:
        pass


class Writer2:
    def write(self, a: str, b: str):
        pass


def make_writer2(w: _Writer2):
    pass


make_writer2(Writer2())


class _Writer3(Protocol):
    def write(self, a: str, b: str) -> object:
        pass


class Writer3:
    def write(self, __a: str, b: str):
        pass


def make_writer3(w: _Writer3):
    pass


# This should generate an error because the source function is positional-only.
make_writer3(Writer3())


class _Writer4(Protocol):
    def write(self, __a: str, b: str) -> object:
        pass


class Writer4:
    def write(self, a: str, b: str):
        pass


def make_writer4(w: _Writer4):
    pass


make_writer4(Writer4())



# This sample tests the reporting of errors for ClassVar in contexts
# where it is not allowed.

from typing import Annotated, Any, ClassVar, Final, Generic, TypeAlias, TypeVar
from typing_extensions import Self  # pyright: ignore[reportMissingModuleSource]

# This should generate an error.
x: ClassVar[int] = 3

T = TypeVar("T")

# This should generate an error.
TA1: TypeAlias = ClassVar[str]


class Foo(Generic[T]):
    x: ClassVar[int] = 3

    # This should generate an error.
    y: Final[ClassVar[int]] = 3

    # This should generate an error.
    z: list[ClassVar[int]] = []

    # This should generate an error because TypeVars cannot
    # be used in a ClassVar.
    illegal1: ClassVar[list[T]]

    # This should generate an error because TypeVars cannot
    # be used in a ClassVar.
    illegal2: ClassVar[T]

    # This should generate an error because Final cannot be
    # used with a ClassVar.
    illegal3: ClassVar[Final] = 0

    # This should generate an error because Final cannot be
    # used with a ClassVar. A second error is generated because
    # Final[int] is not interpreted as a valid type.
    illegal4: ClassVar[Final[int]] = 0

    ok1: ClassVar[list]
    ok2: ClassVar[list[Any]]
    ok3: Annotated[ClassVar[list[Self]], ""]

    # This should generate an error.
    def func1(self, a: ClassVar[int]):
        # This should generate an error.
        x: ClassVar[str] = ""

        # This should generate an error.
        self.xx: ClassVar[str] = ""

    # This should generate an error.
    def func2(self) -> ClassVar[int]:
        return 3



# This sample tests the case that exercises some of the heuristics that
# determine whether a solved TypeVar should retain a literal type.

from typing import Callable, Generic, Literal, TypeVar

FileChanges = dict[str, Literal["created", "edited", "removed"]]

changes: FileChanges = {}
changes.update({filename: "removed" for filename in ["foo.py", "bar.py"]})

_S = TypeVar("_S")
_T = TypeVar("_T")


class ClassA(Generic[_T]):
    pass


TA1 = Callable[[ClassA[_T]], None]


def func1(value: _T) -> TA1[_T]:
    def ret(ctx: ClassA[_T]) -> None:
        pass

    return ret


def func2() -> TA1[bool]:
    return func1(True)


def func3(value: _T) -> Callable[[_T], None]: ...


x: Callable[[tuple[bool]], None] = func3((True,))


def func4(v: _T, f: Callable[[_T], None]): ...


def func5(v: Literal[1, 2], f: Callable[[Literal[1, 2]], None]):
    func4(v, f)


class ClassB(Generic[_S, _T]):
    left: _S
    right: _T


def func6(s: _S, t: _T) -> ClassB[_S, _T]: ...


def func7(t: _T, f: Callable[[ClassB[_T, Literal[2]]], None]) -> None:
    return f(func6(t, 2))


def func8(a: _T, b: Callable[[list[_T]], None]) -> _T:
    return a


def func9(v: Callable[[list[int]], None]):
    func8(b=v, a=1)

    func8(a=1, b=v)



# This sample tests the pre-3.8 position-only parameter convention.


from typing import Any


def func1(__a: int, __b: int, c: int) -> None: ...


func1(1, 2, c=3)

# This should generate an error because __b is position-only.
func1(1, __b=2, c=3)


# This should generate an error because a position-only parameter cannot
# follow a non-position-only parameter.
def func2(a: int, __b: int) -> None: ...


def func3(a: int, *args: Any, __b: int) -> None: ...


func3(a=1, __b=2)


def func4(a: int, /, __b: int) -> None: ...


func4(1, __b=2)


class A:
    def m1(self, __a: int, b: int) -> None:
        pass

    # This should generate an error.
    def m2(self, a: int, __b: int) -> None:
        pass

    @classmethod
    def c1(cls, __a, int, b: int) -> None:
        pass

    @classmethod
    # This should generate an error.
    def c2(cls, a, int, __b: int) -> None:
        pass

    @staticmethod
    def s1(__a: int, b: int) -> None:
        pass

    @staticmethod
    # This should generate an error.
    def s2(a: int, __b: int) -> None:
        pass



from typing import Any, assert_type, runtime_checkable, Protocol, Iterable, Iterator, MutableMapping, Reversible, Callable, TypeIs
from types import FunctionType


class Covariant[T]:
    def foo(self, other: object):
        if isinstance(other, Covariant):
            assert_type(other, Covariant[Any])

    def bar(self) -> T: ...


class InheritCovariant[T](Covariant[T]):
    def baz(self, other: Covariant[int]) -> None:
        if isinstance(other, InheritCovariant):
            assert_type(other, InheritCovariant[int])


class CovariantByDefault[T]:
    """by default if there are no usages of a type var on a class, it's treated as covariant.
    imo this should be an error. see https://github.com/DetachHead/basedpyright/issues/744"""
    def foo(self, other: object):
        if isinstance(other, CovariantByDefault):
            assert_type(other, CovariantByDefault[Any])


class CovariantWithBound[T: int | str]:
    def foo(self, other: object):
        if isinstance(other, CovariantWithBound):
            assert_type(other, CovariantWithBound[Any])

    def bar(self) -> T: ...


class Contravariant[T]:
    def foo(self, other: object):
        if isinstance(other, Contravariant):
            assert_type(other, Contravariant[Any])

    def bar(self, other: T): ...


class ContravariantWithBound[T: int | str]:
    def foo(self, other: object):
        if isinstance(other, ContravariantWithBound):
            assert_type(other, ContravariantWithBound[Any])

    def bar(self, other: T): ...


class Invariant[T]:
    """make sure invariant doesn't think it knows the type param - narrowing to invariant isn't safe """
    def foo(self, other: object):
        if isinstance(other, Invariant):
            assert_type(other, Invariant[Any])  # Unknown

    def bar(self, other: T) -> T: ...


class InvariantWithBound[T: float | bytes]:
    """make sure invariant doesn't think it knows the type param - narrowing to invariant isn't safe """
    def foo(self, other: object):
        if isinstance(other, Invariant):
            assert_type(other, Invariant[Any])  # Unknown

    def bar(self, other: T) -> T: ...


def foo(value: object):
    match value:
        case Iterable():
            assert_type(value, Iterable[Any])


class AnyOrUnknown:
    def __init__(self, value):
        """arguments in `__init__` get turned into fake type vars if they're untyped, so we need to handle this case.
        see https://github.com/DetachHead/basedpyright/issues/746"""
        if isinstance(value, Iterable):
            assert_type(value, Iterable[Any])

    def any(self, value: Any):
        if isinstance(value, Iterable):
            assert_type(value, Iterable[Any])

    def match_case(self, value: Any):
        match value:
            case Iterable():
                assert_type(value, Iterable[Any])

    def unknown(self, value):
        if isinstance(value, Iterable):
            assert_type(value, Iterable[Any])

    def partially_unknown(self, value=None):
        if isinstance(value, Iterable):
            assert_type(value, Iterable[Any])


def goo[KT, VT](self: MutableMapping[KT, VT]) -> Iterator[KT]:
    assert isinstance(self, Reversible)
    return reversed(self)

class Constraints[T: (int, str), U: (int, str), V: int]:
    ...
 
def _(value: object):
    if isinstance(value, Constraints):
        assert_type(value, Constraints[Any, Any, Any])

@runtime_checkable
class Foo[T: (int, str)](Protocol):
    def asdf(self): ...

def _(
    value: str | Foo[str],
):
    if isinstance(value, Foo):
        assert_type(value, Foo[str])

def _(f: Callable[[], None]):
    if isinstance(f, staticmethod):
        assert_type(f, staticmethod[..., Any])

class CallableProtocol[**P, T](Protocol):
    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> T: ...


def _(f: CallableProtocol[[], None]):
    if isinstance(f, staticmethod):
        assert_type(f, staticmethod[[], None])

def _(f: Callable[[int], str]):
    if isinstance(f, FunctionType):
        assert_type(f, FunctionType)
        
def takes_arg(value: object) -> TypeIs[Callable[[int], None]]: ...

def _(value: Callable[[], None] | Callable[[int], None]):
    if takes_arg(value):
        assert_type(value, Callable[[int], None])


# test for an upstream bug that i accidentally fixed
# https://github.com/DetachHead/basedpyright/issues/452
class Bar:
    ...
class Baz(Bar):
    def __call__(self): ...
def _[T, **P](value: Callable[P, T]):
    if isinstance(value, Bar):
        pass


# This sample tests variadic TypeVar matching for unions.

# Enable experimental features to support Union[*Ts].
# pyright: enableExperimentalFeatures=true

from typing import TypeVar, Union
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    TypeVarTuple,
    Unpack,
)


_T = TypeVar("_T")
_Xs = TypeVarTuple("_Xs")
_Ys = TypeVarTuple("_Ys")


def func1(x: Union[Unpack[_Xs]]) -> Union[Unpack[_Xs]]: ...


def func2(x: Union[Unpack[_Xs], Unpack[_Ys]]) -> Union[Unpack[_Xs], Unpack[_Ys]]: ...


def func3(x: Union[int, Unpack[_Xs]]) -> Union[Unpack[_Xs]]: ...


def func4(x: Union[_T, Unpack[_Xs]]) -> Union[_T, Unpack[_Xs]]: ...


def func5(x: Union[Unpack[_Xs]], *args: Unpack[_Xs]) -> Union[Unpack[_Xs]]: ...


def func6(*args: Unpack[_Xs]) -> Union[Unpack[_Xs]]: ...


def func7(a: list[Union[Unpack[_Xs]]]) -> Union[Unpack[_Xs]]: ...


def test1(a: int, b: str, c: list[int], d: Union[complex, str]):
    v1_1 = func1(a)
    reveal_type(v1_1, expected_text="int")

    v1_2 = func1(d)
    reveal_type(v1_2, expected_text="complex | str")

    # ---------

    # This behavior isn't defined by PEP 646, but neither
    # did PEP 484 define the behavior for multiple (non-
    # variadic) TypeVar matching within a Union. So behavior
    # is likely to vary between type checkers here.
    v2_1 = func2(a)
    reveal_type(v2_1, expected_text="int")

    v2_2 = func2(d)
    reveal_type(v2_2, expected_text="complex | str")

    # ---------

    v3_1 = func3(a)
    reveal_type(v3_1, expected_text="Unknown")

    # This should generate an error
    v3_2 = func3(d)

    v3_3 = func3(b)
    reveal_type(v3_3, expected_text="str")

    # ---------

    # This behavior isn't defined by PEP 646 or PEP 484.
    v4_1 = func4(a)
    reveal_type(v4_1, expected_text="int")

    v4_2 = func4(d)
    reveal_type(v4_2, expected_text="complex | str")

    # ---------

    v5_1 = func5(a)
    reveal_type(v5_1, expected_text="int")

    v5_2 = func5(a, a)
    reveal_type(v5_2, expected_text="int")

    # This should generate an error
    v5_3 = func5(a, b)

    # This should generate an error
    v5_4 = func5(a, b, c)

    # ---------

    v6_1 = func6(a)
    reveal_type(v6_1, expected_text="int")

    v6_2 = func6(a, b)
    reveal_type(v6_2, expected_text="int | str")

    v6_3 = func6(a, b, d)
    reveal_type(v6_3, expected_text="int | str | complex")

    v6_4 = func6()
    reveal_type(v6_4, expected_text="Never")

    # ---------

    v7_1 = func7([a])
    reveal_type(v7_1, expected_text="int")

    x: list[Union[int, str]] = [a, b]
    v7_2 = func7(x)
    reveal_type(v7_2, expected_text="int | str")

    v7_3 = func7([a, b, d])
    reveal_type(v7_3, expected_text="int | str | complex")



# This sample tests the case where a __getattr__ method override
# differentiates based on the name of the accessed member.

from typing import Any, overload, Literal


class Obj:
    @overload
    def __getattr__(self, name: Literal["foo"]) -> int: ...

    @overload
    def __getattr__(self, name: Literal["bar"]) -> str: ...

    def __getattr__(self, name: str) -> Any:
        if name == "foo":
            return 1
        return "1"

    def __setattr__(self, name: str, value: str) -> None:
        pass


obj = Obj()
b1 = obj.foo
reveal_type(b1, expected_text="int")
b2 = getattr(obj, "foo")
reveal_type(b2, expected_text="Any")

c1 = obj.bar
reveal_type(c1, expected_text="str")
c2 = getattr(obj, "bar")
reveal_type(c2, expected_text="Any")

# This should generate two errors.
d1 = obj.bogus

obj.foo = ""
obj.bar = ""

# This should generate an error.
obj.foo = 1

# This should generate an error.
obj.other = 1

# This should generate an error.
del obj.foo



# This sample tests a variety of unicode characters including those that
# require two-code (surrogate) forms.

# Old Italic
𐌎𐌘𐌟𐌁 = 42

# Egyptian hieroglyphs
𓃘𓐭𓇀𓅨𓆙 = 2

# Linear B Ideograms
𐂂𐃪𐃯 = ""

# Cuneiform
𒀟𒀕𒀰𒁜𒂐𒄊 = ""

# Old Persian
𐎠𐏊𐏏 = 3

# Lydian
𐤢𐤷𐤬𐤮 = 4

# Phoenician
𐤔𐤑𐤇 = 4

# Nabataean
𐢖𐢊ﬗ = 0

# CJK ideographs
㐀䶿一鿿𠀀𪛖𪜀𫜴𫝀𫠝𫠠𬺡𬺰𮯠𣎴 = 1

# This should generate an error because "𐢭" is outside the range of
# characters supported by the Python standard.
𐢭 = 0

# Other surrogate characters
𝓐 = 3
𝙰 = 4
𝚫 = 3
𞡏 = 4
𞥁 = 0





# This sample tests the case where a higher-order function receives
# a generic function as an argument, and the type of the generic
# function depends on one of the other arguments passed to the
# higher-order function. This shouldn't depend on the order the
# arguments are passed.


from typing import Protocol, TypeVar


T = TypeVar("T")


class Proto1(Protocol[T]):
    def method(self, v: T) -> T: ...


class Impl1:
    def method(self, v: T) -> T: ...


def func1(a: Proto1[T], b: T) -> T: ...


v1 = func1(a=Impl1(), b="abc")
reveal_type(v1, expected_text="str")

v2 = func1(b="abc", a=Impl1())
reveal_type(v2, expected_text="str")

v3 = func1(a=Impl1(), b=1)
reveal_type(v3, expected_text="int")

v4 = func1(b=1, a=Impl1())
reveal_type(v4, expected_text="int")



# This sample tests proper scopes for nested classes.


class A:
    a = 5

    class B:
        # This should generate an error
        b = a

        class C:
            # This should generate an error
            c = a

            # This should generate an error
            d = b



# This sample tests a member access when the metaclass implements a descriptor
# protocol.

from typing import Any, TypeVar, overload


T = TypeVar("T")


class MetaClass(type):
    @overload
    def __get__(self: type[T], instance: None, owner: Any) -> type[T]: ...

    @overload
    def __get__(self: type[T], instance: object, owner: Any) -> T: ...

    def __get__(self: type[T], instance: object | None, owner: Any) -> type[T] | T:
        if instance is None:
            return self
        return self()


class A(metaclass=MetaClass): ...


class B:
    a = A


reveal_type(B.a, expected_text="type[A]")
reveal_type(B().a, expected_text="A")



# This sample tests the case where a recursive type alias contains
# a `Sequence` that potentially overlaps with str (which derives from
# Sequence[str]).

from typing import Sequence


NestedIntList = int | Sequence["NestedIntList"]


def func1() -> NestedIntList:
    result: NestedIntList = 1
    for _ in range(1):
        result = [result]
    return result


def func2() -> NestedIntList:
    # This should generate an error.
    result: NestedIntList = ""
    for _ in range(1):
        result = [result]
    return result



# generateUnicodeTables.py
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.
#
# Generates the content of unicode.ts based on the official Unicode
# character database.

import sys
import urllib.request
from io import TextIOWrapper


class Character:
    def __init__(self, code: int, category: str, *, end: int | None = None):
        self.code = code
        self.category = category

        self.hasSurrogate = code > 0xFFFF
        if self.hasSurrogate:
            unicodeChar = chr(code)
            utf16 = unicodeChar.encode("utf-16")
            rawHex = utf16.hex()
            hex = rawHex[4:]

            self.highSurrogate = int(hex[2:4] + hex[0:2], base=16)
            self.lowSurrogate = int(hex[6:8] + hex[4:6], base=16)


class CharacterRange:
    def __init__(self, start: Character, end: Character):
        self.start = start
        self.end = end


def downloadUnicodeData(unicodeVersion: str) -> str:
    url = f"https://www.unicode.org/Public/{unicodeVersion}.0/ucd/UnicodeData.txt"
    (path, _) = urllib.request.urlretrieve(url)
    return path


def parseFile(filePath: str) -> list[Character]:
    with open(filePath, "r") as reader:
        lines = reader.readlines()
        chars: list[Character] = []
        for i in range(len(lines)):
            line = lines[i]
            splitOnSemicolon = line.split(";")
            charCode = int(splitOnSemicolon[0], base=16)
            category = splitOnSemicolon[2]

            if splitOnSemicolon[1].endswith(", First>"):
                # Legacy range syntax
                # D800;<Non Private Use High Surrogate, First>;Cs;0;L;;;;;N;;;;;
                # DB7F;<Non Private Use High Surrogate, Last>;Cs;0;L;;;;;N;;;;;
                nextLine = lines[i + 1]
                nextSplitOnSemicolon = nextLine.split(";")
                nextCharCode = int(nextSplitOnSemicolon[0], base=16)
                for ord in range(charCode, nextCharCode + 1):
                    chars.append(Character(ord, category))
            elif splitOnSemicolon[1].endswith(", Last>"):
                continue
            else:
                chars.append(Character(charCode, category))

        return chars


# Given a collection of characters, returns a list of ranges of contiguous
# characters. Contiguous means that the character codes are sequential with
# no gaps and the characters all have the same category. For character codes
# greater than 0xFFFF, contiguous means that the high surrogate is the same
# and the low surrogate values are sequential with no gaps. So, two character
# codes might be sequential numerically but have different high surrogates,
# and therefore would not be members of the same range.
def getSurrogateRanges(chars: list[Character]) -> list[CharacterRange]:
    surrogateRanges: list[CharacterRange] = []

    consecutiveRangeStartChar: Character | None = None
    previousChar: Character | None = None
    for char in chars:
        if not consecutiveRangeStartChar:
            consecutiveRangeStartChar = char

        if previousChar:
            if not previousChar.hasSurrogate and not char.hasSurrogate:
                if (
                    char.code == previousChar.code + 1
                    and char.category == previousChar.category
                ):
                    pass
            elif not previousChar.hasSurrogate and char.hasSurrogate:
                consecutiveRangeStartChar = char
            else:
                if (
                    char.highSurrogate == previousChar.highSurrogate
                    and char.lowSurrogate == previousChar.lowSurrogate + 1
                    and char.category == previousChar.category
                ):
                    pass
                else:
                    surrogateRanges.append(
                        CharacterRange(consecutiveRangeStartChar, previousChar)
                    )
                    consecutiveRangeStartChar = char

        previousChar = char

    return surrogateRanges


# Write out a table of all character codes within the specified category. These are
# the full hex character codes (Unicode code points) not surrogate values. Sequential
# ranges of character codes are written as arrays of two numbers (start and end) to
# save space.
def writeRangeTable(writer: TextIOWrapper, category: str, chars: list[Character]):
    chars = [ch for ch in chars if ch.category == category]

    writer.write("\n")
    writer.write(f"export const unicode{category}: UnicodeRangeTable = [\n")

    consecutiveRangeStartChar: Character | None = None
    for i in range(len(chars)):
        char = chars[i]

        if not consecutiveRangeStartChar:
            consecutiveRangeStartChar = char

        if i + 1 >= len(chars) or chars[i + 1].code != char.code + 1:
            if consecutiveRangeStartChar.code == char.code:
                writer.write(f"    0x{consecutiveRangeStartChar.code:04x},\n")
            else:
                writer.write(f"    [0x{consecutiveRangeStartChar.code:04x}, 0x{char.code:04x}],\n")

            consecutiveRangeStartChar = None

    writer.write("];\n")


# Write out a table of all characters within the specified category using their UTF-16
# values. Characters are grouped by high surrogate value. Sequential ranges of low
# surrogate values are written as arrays of two numbers (start and end) to save space.
def writeSurrogateRangeTable(
    writer: TextIOWrapper, category: str, surrogateRanges: list[CharacterRange]
):
    surrogateRanges = [r for r in surrogateRanges if r.start.category == category]

    if len(surrogateRanges) == 0:
        return

    writer.write("\n")
    writer.write(
        f"export const unicode{category}Surrogate: UnicodeSurrogateRangeTable = {{\n"
    )

    previousCharRange: CharacterRange | None = None
    for charRange in surrogateRanges:
        if (
            previousCharRange
            and charRange.start.highSurrogate != previousCharRange.start.highSurrogate
        ):
            writer.write("    ],\n")
            previousCharRange = None

        if not previousCharRange:
            writer.write(f"    0x{charRange.start.highSurrogate:04x}: [\n")
            previousCharRange = charRange

        if charRange.start.lowSurrogate == charRange.end.lowSurrogate:
            writer.write(f"        0x{charRange.start.lowSurrogate:04x}, // 0x{charRange.start.code:04X}\n")
        else:
            writer.write(
                f"        [0x{charRange.start.lowSurrogate:04x}, 0x{charRange.end.lowSurrogate:04x}], // 0x{charRange.start.code:04X}..0x{charRange.end.code:04X}\n"
            )

    writer.write("    ],\n")
    writer.write("};\n")


unicodeVersion = "16.0" if len(sys.argv) <= 1 else sys.argv[1]
path = downloadUnicodeData(unicodeVersion)
chars = parseFile(path)
surrogateRanges = getSurrogateRanges(chars)

with open("packages/pyright-internal/src/parser/unicode.ts", "w") as writer:
    writer.write(
        f"""/*
 * unicode.ts
 * Copyright (c) Microsoft Corporation.
 * Licensed under the MIT license.
 *
 * Tables that encode Unicode character codes for various Unicode-
 * defined categories used in the Python spec.
 *
 * Generated by build/generateUnicodeTables.py from the UnicodeData.txt
 * metadata file for Unicode {unicodeVersion}.
 */

export type UnicodeRange = [number, number] | number;
export type UnicodeRangeTable = UnicodeRange[];
export type UnicodeSurrogateRangeTable = {{ [surrogate: number]: UnicodeRange[] }};
"""
    )

    for category in ["Lu", "Ll", "Lt", "Lo", "Lm", "Nl", "Mn", "Mc", "Nd", "Pc"]:
        writeRangeTable(writer, category, chars)
        writeSurrogateRangeTable(writer, category, surrogateRanges)



# This sample tests the case where a class uses auto-variance but derives
# from a class that does not.

from typing import Generic, TypeVar

T = TypeVar("T")
T_co = TypeVar("T_co", covariant=True)
T_contra = TypeVar("T_contra", contravariant=True)


class Parent_Invariant(Generic[T]):
    pass


class ShouldBeInvariant[T](Parent_Invariant[T]):
    pass


# This should generate an error.
a1: ShouldBeInvariant[int] = ShouldBeInvariant[float]()

# This should generate an error.
a2: ShouldBeInvariant[float] = ShouldBeInvariant[int]()


class Parent_Covariant(Generic[T_co]):
    pass


class ShouldBeCovariant[T](Parent_Covariant[T]):
    pass


# This should generate an error.
b1: ShouldBeCovariant[int] = ShouldBeCovariant[float]()

b2: ShouldBeCovariant[float] = ShouldBeCovariant[int]()


class Parent_Contravariant(Generic[T_contra]):
    pass


class ShouldBeContravariant[T](Parent_Contravariant[T]):
    pass


c1: ShouldBeContravariant[int] = ShouldBeContravariant[float]()

# This should generate an error.
c2: ShouldBeContravariant[float] = ShouldBeContravariant[int]()



# This sample tests annotated types on global variables.

# This should generate an error because the declared
# type below does not match the assigned type.
glob_var1 = 4

# This should generate an error because the declared
# type doesn't match the later declared type.
glob_var1 = Exception()  # type: str

glob_var1 = Exception()  # type: Exception

# This should generate two errors: one because the assigned
# type doesn't match the declared type, and the other because
# it's a redeclaration
glob_var1 = "hello"  # type: Exception

# This should generate an error.
glob_var2 = 5


def func1():
    global glob_var1
    global glob_var2

    # This should generate an error.
    glob_var1 = 3

    glob_var2 = "hello"  # type: str



# This sample tests the reporting of incorrect TypeVar usage within
# a generic function. A TypeVar must appear at least twice to be
# considered legitimate, except for the return position.

# pyright: reportInvalidTypeVarUse=true

from typing import AnyStr, Callable, Generic, overload
from typing_extensions import TypeVar  # pyright: ignore[reportMissingModuleSource]


_T = TypeVar("_T")
_S = TypeVar("_S")


class A(Generic[_T]):
    def m1(self, v1: _T) -> None: ...

    # This should generate an error because _S
    # is a local typeVar and appears only once.
    def m2(self, v1: _S) -> None: ...

    # This should generate an error because _S
    # is a local typeVar and appears only once.
    def m3(self, v1: _T) -> _S: ...


# This should generate an error because _T
# is a local typeVar and appears only once.
def f1(v1: _T) -> None: ...


def f2(v1: _T, v2: list[_T]) -> None: ...


def f3(v1: _T) -> _T: ...


def f4() -> dict[_T, _T]: ...


# This should not generate an error because _T
# is a local typeVar and appears only once, but in a return position.
def f5() -> list[_T]: ...


_T_Bound = TypeVar("_T_Bound", bound=int)
_T_Constrained = TypeVar("_T_Constrained", int, str)


# Constrained TypeVars are exempt.
def f6(v1: _T_Constrained): ...


# Bound TypeVars are not exempt.
def f7(v1: _T_Bound): ...


# Bound TypeVars as type arguments are exempt when used in an
# input parameter annotation.
def f8(v1: list[_T_Bound]): ...


# Bound TypeVars as type arguments are not exempt when used in a
# return annotation. well... they are, so.
def f9() -> list[_T_Bound]: ...


# TypeVars used as type args to a generic type alias are exempt.
MyCallable = Callable[[_T], _T]


def f10() -> MyCallable[_T]: ...


# This should generate an error because AnyStr can go unsolved.
def f11(x: AnyStr = ...) -> AnyStr: ...


# This should generate an error because AnyStr can go unsolved.
def f12(x: AnyStr = ...) -> list[AnyStr]: ...


def f13(x: AnyStr = ...) -> AnyStr | None: ...


def f14(x: AnyStr = "") -> AnyStr: ...


# This should generate an error because AnyStr can go unsolved.
def f15(x: AnyStr = ...) -> list[AnyStr] | None: ...


class B(Generic[AnyStr]):
    # This should generate an error because AnyStr can go unsolved.
    def __init__(self, *, mode: AnyStr = ...) -> None: ...


class C(Generic[AnyStr]):
    def __init__(self, *, mode: AnyStr = "") -> None: ...


@overload
def f16(default: int = ...) -> list[int]: ...


@overload
def f16(default: _T) -> list[_T]: ...


def f16(default: _T = ...) -> list[int] | list[_T]: ...


class ClassA(Generic[_T]):
    # This should generate an error because _T can go unsolved.
    def __init__(self, x: _T = ...) -> None: ...


_T2 = TypeVar("_T2", default=int)


class ClassB(Generic[_T2]):
    def __init__(self, x: _T2 = ...) -> None: ...


# This should generate an error because _T appears only once.
def f17(
    arg,  # type: _T
):  # type: (...) -> int
    return 1


def f18(
    arg,  # type: _T
):  # type: (...) -> _T
    return arg


# This should generate an error because _T appears only once.
def f19(
    arg,
):  # type: (_T) -> int
    return 1


def f20(
    arg,  # type: _T
):  # type: (...) -> _T
    return arg



# This sample verifies that the type checker allows access
# to class variables provided by a metaclass.

from enum import Enum
from typing import Mapping


class Fruit(Enum):
    apple = 1
    orange = 2
    pear = 3


def requires_fruit_mapping(a: Mapping[str, Fruit]):
    pass


requires_fruit_mapping(Fruit.__members__)

aaa = len(Fruit)

for i in Fruit:
    print(i)


class Meta(type):
    inst_var1: int

    def __init__(self):
        self.inst_var1 = 1

    def method1(cls, some_param: str):
        pass


class MyClass1(metaclass=Meta):
    pass


MyClass1.method1("some argument")
reveal_type(MyClass1.inst_var1, expected_text="int")


class MyClass2(metaclass=Meta):
    # This should generate an error
    inst_var1 = ""


class MyClass3(metaclass=Meta):
    def __init__(self):
        # This should generate an error
        self.inst_var1 = ""



# This sample tests that unhashable user classes are detected as unhashable.

# pyright: reportIncompatibleMethodOverride=false


class A: ...


s1 = {A()}
d1 = {A(): 100}


class B:
    def __eq__(self, other): ...


# Both of these should generate an error because a class that
# defines __eq__ but not __hash__ is not hashable.
s2 = {B()}
d2 = {B(): 100}


class C:
    __hash__: None = None


class D(B, C): ...


# Both of these should generate an error because B is unhashable.
s3 = {D()}
d3 = {D(): 100}


class E:
    def __hash__(self): ...


class F(D, E): ...


# Both of these should generate an error because D is unhashable.
s4 = {F()}
d4 = {F(): 100}


class G(E, D): ...


s5 = {G()}
d5 = {G(): 100}



# This sample verifies that an error is returned if an inner function
# doesn't use P.args and P.kwargs in its parameter list but is returned
# by an outer function that uses P in its return type.

from typing import Callable, Concatenate
from typing_extensions import ParamSpec  # pyright: ignore[reportMissingModuleSource]

P = ParamSpec("P")


def func1(func: Callable[P, int]) -> Callable[P, int]:
    def inner_func(x: int) -> int:
        # This should generate a type error.
        return func()

    # This should generate a type error.
    return inner_func


def func2(
    func: Callable[Concatenate[int, P], int],
) -> Callable[Concatenate[int, P], int]:
    def inner_func(x: int) -> int:
        # This should generate a type error.
        return func(x)

    # This should generate a type error.
    return inner_func



# This sample tests the case where a generic class has a constructor that
# supplies the type arguments via a callable which is itself generic.


from typing import Callable, Generic, Sequence, TypeVar

T = TypeVar("T")
V = TypeVar("V", bound=object)
V_co = TypeVar("V_co", covariant=True)
U = TypeVar("U", bound=object)


class Result(Generic[V]):
    pass


ParseFn = Callable[[Sequence[T], int, int], Result[V]]


class Parser(Generic[T, V_co]):
    def fmap1(self, fn: Callable[[V_co], U]) -> "Parser[T, U]":
        def fmap2(stream: Sequence[T], pos: int, bt: int) -> Result[U]:
            raise NotImplementedError()

        reveal_type(FnParser(fmap2), expected_text="FnParser[T@Parser, U@fmap1]")
        return FnParser(fmap2)


class FnParser(Parser[T, V_co]):
    def __init__(self, fn: ParseFn[T, V_co]):
        self._fn = fn



# This sample tests various conditions with AND and OR operators.


def func1() -> bool: ...


def func2() -> int: ...


def func3() -> str: ...


reveal_type(func1() and func2(), expected_text="int | Literal[False]")
reveal_type(func1() and func3(), expected_text="str | Literal[False]")
reveal_type(func2() and func1(), expected_text="bool | Literal[0]")
reveal_type(func3() and func1(), expected_text="bool | Literal['']")

reveal_type(func1() or func2(), expected_text="int | Literal[True]")
reveal_type(func1() or func3(), expected_text="str | Literal[True]")
reveal_type(func2() or func1(), expected_text="int | bool")
reveal_type(func3() or func1(), expected_text="str | bool")


class ClassA: ...


class ClassB: ...


# This should generate an error because binary operators are not allowed
# in type annotations.
def func4(a: ClassA and ClassB): ...


# This should generate an error because binary operators are not allowed
# in type annotations.
def func5(a: ClassA or ClassB): ...



# This sample tests the case where a callback protocol uses a default argument
# but the corresponding callable does not or vice versa.

from typing import Protocol


# Callback with positional parameter with default arg value.
class Callback1(Protocol):
    def __call__(self, path: str = ...) -> str: ...


# Callback with positional parameter without default arg value.
class Callback2(Protocol):
    def __call__(self, path: str) -> str: ...


def func1_1(path: str = "") -> str: ...


def func1_2(path: str) -> str: ...


val1_1: Callback1 = func1_1

# This should generate an error.
val1_2: Callback1 = func1_2

val2_1: Callback2 = func1_1

val2_2: Callback2 = func1_2


# Callback with keyword parameter with default arg value.
class Callback3(Protocol):
    def __call__(self, *, path: str = ...) -> str: ...


# Callback with keyword parameter without default arg value.
class Callback4(Protocol):
    def __call__(self, *, path: str) -> str: ...


def func3_1(*, path: str = "") -> str: ...


def func3_2(*, path: str) -> str: ...


val3_1: Callback3 = func3_1

# This should generate an error.
val3_2: Callback3 = func3_2

val4_1: Callback4 = func3_1

val4_2: Callback4 = func3_2



# This sample tests support for PEP 696 (default types for TypeVars)
# when used to define generic functions and with defaults type
# expressions that refer to other type variables.

from typing import Generic, Self
from typing_extensions import TypeVar  # pyright: ignore[reportMissingModuleSource]

T1 = TypeVar("T1", default=str)
T2 = TypeVar("T2", default=list[T1])


def func1(x: T1, y: int | T2 = 0) -> T2 | list[T1]: ...


v1_1 = func1("hi", 3.4)
reveal_type(v1_1, expected_text="float | list[str]")

v1_2 = func1("")
reveal_type(v1_2, expected_text="list[str]")


# This should generate an error because T1 depends on T2.
def func2(x: T2, y: T1) -> list[T1 | T2]: ...


T3 = TypeVar("T3", default=int)


class ClassA(Generic[T3]):
    def __init__(self, value: T3):
        self.value = value

    def func1(self, value: T3) -> Self:
        self.value = value
        return self


T4 = TypeVar("T4", default=int)
T5 = TypeVar("T5", default=T4)


class ClassB(Generic[T4, T5]):
    @property
    def x(self) -> T4: ...

    @property
    def y(self) -> T5: ...


b1 = ClassB()
reveal_type(b1.x, expected_text="int")
reveal_type(b1.y, expected_text="int")


T6 = TypeVar("T6", default=int)
T7 = TypeVar("T7", default=T6)
T8 = TypeVar("T8", default=int | None)


class ClassC(Generic[T6, T7, T8]):
    def __new__(cls, x: T7, /) -> Self: ...

    def method1(self) -> T7: ...



# This sample tests the return type inference for a generator.

from typing import Awaitable, Generator, TypeVar

S = TypeVar("S")
T = TypeVar("T")


def func1() -> Generator[int, int, str]:
    yield 1
    return "done"


def func2() -> Generator[int, int, None]:
    # This should generate an error because yield is not allowed
    # from within a list comprehension.
    x = [(yield from func1()) for lel in range(5)]

    v1 = yield from func1()
    reveal_type(v1, expected_text="str")

    v2 = yield 4
    reveal_type(v2, expected_text="int")


def func3():
    [x for x in (yield [[[1]], [[2]], [[3]]]) for y in x]

    # This should generate an error.
    [x for x in [[[1]], [[2]], [[3]]] for y in (yield x)]


class ClassA[S, T](Awaitable[T]):
    def __init__(self, val: S) -> None:
        self.val = val

    def __await__(self) -> Generator[S, T, T]:
        z = yield self.val
        reveal_type(z, expected_text="T@ClassA")
        return z



# This sample tests the case where Annotated is used with deferred
# annotation evaluation.

from __future__ import annotations
from typing import Annotated


v1: Annotated[str, ClassA, func1(), v2[0]] = ""

v2 = [1, 2, 3]


class ClassA: ...


def func1(): ...



# This sample tests wildcard imports.

from .import5 import *

a = foo

# This should generate an error because there is no
# __all__ assignment, and names starting with a double underscore
# should not be imported in a wildcard.
b = __foo

c = bar

# This should generate an error because there is no __all__ assignment
# and names starting with a single underscore should not be imported
# in a wildcard.
d = _bar



# This sample tests Pyright's handling of recursive type aliases.

from typing import Mapping, TypeVar, Union

MyTree = list[Union["MyTree", int]]

t1: MyTree = [1, 2, 3, [3, 4], [[3], 5]]

# This should generate an error because a str is not allowed.
t2: MyTree = [3, ""]

# This should generate an error because a str is not allowed.
t3: MyTree = [1, 2, 3, [3, 4], [3, 4, 5, [3, "4"]]]

_T = TypeVar("_T")
GenericUnion = Union[int, _T]

i1: GenericUnion[str] = "hi"
i1 = 3

i2: GenericUnion[float] = 3
# This should generate an error because str isn't compatible.
i2 = "hi"

Foo = Union[bool, list["Foo"], dict["Foo", "Foo"]]

bar1: Foo = [True, [True, False]]
bar2: Foo = [True, [True], {True: False}]
bar4: Foo = {True: [False]}

# These should generate errors.
baz1: Foo = [True, ["True", False]]
baz2: Foo = [True, [True], {True: "False"}]
baz4: Foo = {True: ["False"]}

Json = Union[None, int, str, float, list["Json"], dict[str, "Json"]]

# This should generate an error
a1: Json = {"a": 1, "b": 3j}

# This should generate an error
a2: Json = [2, 3j]

RecursiveTuple = Union[str | int, tuple["RecursiveTuple", ...]]


b1: RecursiveTuple = (1, 1)
b2: RecursiveTuple = (1, "1")
b3: RecursiveTuple = (1, "1", 1, "2")
b4: RecursiveTuple = (1, ("1", 1), "2")
b5: RecursiveTuple = (1, ("1", 1), (1, (1, 2)))

# This should generate an error
b6: RecursiveTuple = (1, ("1", 1), (1, (1, [2])))

# This should generate an error
b7: RecursiveTuple = (1, [1])


RecursiveMapping = Union[str, int, Mapping[str, "RecursiveMapping"]]


c1: RecursiveMapping = 1
c2: RecursiveMapping = "1"
c3: RecursiveMapping = {"1": "1"}
c4: RecursiveMapping = {"1": "1", "2": 1}
c5: RecursiveMapping = {"1": "1", "2": 1, "3": {}}
c6: RecursiveMapping = {"1": "1", "2": 1, "3": {"0": "0", "1": "2", "2": {}}}

# This should generate an error.
c7: RecursiveMapping = {"1": [1]}

# This should generate an error.
c8: RecursiveMapping = {"1": "1", "2": 1, "3": [1, 2]}

# This should generate an error.
c9: RecursiveMapping = {"1": "1", "2": 1, "3": {"0": "0", "1": 1, "2": [1, 2, 3]}}



# This sample tests various forms of subscript expressions for
# syntax and semantic (type) errors.

from typing import TypeVar


_T = TypeVar("_T", list, tuple)

def func1(p1: list[int], p2: _T):
    a1 = p1[0]
    reveal_type(a1, expected_text="int")

    a2 = p1[:]
    reveal_type(a2, expected_text="list[int]")

    a3 = p1[1:]
    reveal_type(a3, expected_text="list[int]")

    a4 = p1[1:2]
    reveal_type(a4, expected_text="list[int]")

    a5 = p1[0:1:3]
    reveal_type(a5, expected_text="list[int]")

    a6 = p1[:3]
    reveal_type(a6, expected_text="list[int]")

    a7 = p1[::]
    reveal_type(a7, expected_text="list[int]")

    a8 = p1[::2]
    reveal_type(a8, expected_text="list[int]")

    # This should generate a syntax error.
    b1 = p1[0:1:3:4]

    # This should generate a syntax error.
    b2 = p1[0:::]

    # This should generate a type error.
    c1 = p1[:,]
    reveal_type(c1, expected_text="Unknown")

    # This should generate a type error.
    c2 = p1[:,:]
    reveal_type(c2, expected_text="Unknown")

    # This should generate a type error.
    c3 = p1[1,]
    reveal_type(c3, expected_text="Unknown")

    d1 = p2[0]
    reveal_type(d1, expected_text="Unknown")





# This sample tests assignment of dataclass fields that use
# the converter parameter described in PEP 712.

from typing import Any, Callable, dataclass_transform


def converter_simple(s: str) -> int: ...


def converter_passThru(x: str | int) -> str | int: ...


def model_field(*, converter: Callable[..., Any]) -> Any: ...


@dataclass_transform(field_specifiers=(model_field,))
class ModelBase: ...


class DC1(ModelBase):
    asymmetric: int = model_field(converter=converter_simple)
    symmetric: str | int = model_field(converter=converter_passThru)


dc1 = DC1("1", 1)

reveal_type(dc1.asymmetric, expected_text="int")
dc1.asymmetric = "2"
reveal_type(
    dc1.asymmetric, expected_text="int"
)  # Asymmetric -- type narrowing should not occur
# This should generate an error because only strs can be assigned to field0.
dc1.asymmetric = 2

reveal_type(dc1.symmetric, expected_text="str | int")
dc1.symmetric = "1"
reveal_type(
    dc1.symmetric, expected_text="Literal['1']"
)  # Symmetric -- type narrowing should occur


reveal_type(DC1.asymmetric, expected_text="int")
DC1.asymmetric = "2"
reveal_type(DC1.asymmetric, expected_text="int")
# This should generate an error because only strs can be assigned to field0.
DC1.asymmetric = 2

reveal_type(DC1.symmetric, expected_text="str | int")
DC1.symmetric = "1"
reveal_type(DC1.symmetric, expected_text="Literal['1']")


class DC2(ModelBase):
    a: dict[str, str] = model_field(converter=dict)


DC2({})
DC2({"": ""})

# This should generate an error.
DC2({"": 1})


class DC3(ModelBase):
    b: tuple[int, ...] = model_field(converter=tuple)


DC3([1, 2, 3])

# This should generate an error.
DC3(["", 1])



# This sample tests the case where a lambda's expression must be
# evaluated multiple times as more type information is gathered
# in the presence of an overloaded method.

# pyright: strict


def func1(keys: list[str]):
    filter(lambda s: s.startswith(""), keys)



# This sample tests the inferred type of async and sync generators.


async def foo() -> int: ...


async def main() -> None:
    v1 = (x for x in [2, 3] if x > 3)
    reveal_type(v1, expected_text="Generator[int, None, None]")

    v2 = (x for x in [2, 3] if await foo())
    reveal_type(v2, expected_text="AsyncGenerator[int, None]")

    v3 = (x for x in [2, 3])
    reveal_type(v3, expected_text="Generator[int, None, None]")

    v4 = (await foo() for _ in [2, 3])
    reveal_type(v4, expected_text="AsyncGenerator[int, None]")

    v5 = ((0, await foo()) for _ in [1, 2])
    reveal_type(v5, expected_text="AsyncGenerator[tuple[int, int], None]")

    v6 = (x for x in [1, 2] if (x, await foo()))
    reveal_type(v6, expected_text="AsyncGenerator[int, None]")



# This sample tests that args and kwargs parameters are
# properly typed.


def function_with_args(*args: str):
    reveal_type(args, expected_text="tuple[str, ...]")


def function_with_kwargs(**kwargs: list[str]):
    reveal_type(kwargs, expected_text="dict[str, list[str]]")



# This sample tests the case where a base class has
# an unknown type and the type is used in an
# isinstance check.

# pyright: reportUnnecessaryIsInstance=true

from typing import TypeVar, Union

# This should generate an error because "dummy" can't be resolved.
# The symbol Document should have an unknown type.
from dummy import Document


class DbModel(Document):
    pass


def func1() -> Union[int, DbModel]:
    return DbModel()


# This should not generate an error even though DbModel is
# derived from an unknown base class.
isinstance(func1(), int)


def func2(obj: object, typ: type):
    return isinstance(obj, typ)


def func3(obj: float):
    if isinstance(obj, float):
        reveal_type(obj, expected_text="float")
    else:
        reveal_type(obj, expected_text="int")


T = TypeVar("T", bound=float)


def func4(t: type[T]):
    if issubclass(t, float):
        reveal_type(t, expected_text="type[float]*")
    else:
        reveal_type(t, expected_text="type[int]*")



# This sample tests various illegal forms of Literal.

from enum import Enum
from pathlib import Path
from typing import Any, Literal, TypeVar

Wrong1 = Literal[3 + 4]

Wrong2 = Literal["foo".replace("o", "b")]

Wrong3 = Literal[4 + 3j]

Wrong4 = Literal[-4 + 2j]

Wrong5 = Literal[(1, "foo", "bar")]

Wrong6 = Literal[{"a": "b", "c": "d"}]

Wrong7 = Literal[Path("abcd")]
T = TypeVar("T")

Wrong8 = Literal[T]

Wrong9 = Literal[3.14]

Wrong10 = Literal[Any]

Wrong11 = Literal[...]


def func(): ...


Wrong12 = Literal[func]
some_variable = "foo"

Wrong13 = Literal[some_variable]


# This should generate two errors.
var1: Literal[3 + 4]

# This should generate two errors.
var2: Literal["foo".replace("o", "b")]

# This should generate two errors.
var3: Literal[4 + 3j]

# This should generate three errors.
var4: Literal[-4 + 2j]

# This should generate an error.
var5: Literal[(1, "foo", "bar")]

# This should generate an error.
var6: Literal[{"a": "b", "c": "d"}]

# This should generate two errors.
var7: Literal[Path("abcd")]

# This should generate an error.
var8: Literal[T]

# This should generate an error.
var9: Literal[3.14]

# This should generate an error.
var10: Literal[Any]

# This should generate an error.
var11: Literal[...]

# This should generate an error.
var12: Literal[func]

# This should generate two errors.
var13: Literal[some_variable]


class Enum1(Enum):
    A = 1
    B = 2

    x: str


a = Enum1.A

# This should generate two errors.
var14: Literal[a]

# This should generate two errors.
var15: Literal[Enum1.x]



# This sample tests that type compatibility between TypedDicts.

from typing import TypedDict, final


class TD0(TypedDict):
    key: str


class TD1(TD0):
    value: str


class TD2(TypedDict):
    key: str
    value: str


v1: TD2 = TD1(key="", value="")
v2: TD1 = TD2(key="", value="")

v3 = [v2]
v4: list[TD2] = v3
v5 = [v1]
v6: list[TD1] = v5


class TD10(TypedDict, total=False):
    key: str


class TD11(TD10):
    value: str


class TD12(TypedDict):
    key: str
    value: str


# This should generate an error.
v10: TD12 = TD11(key="", value="")

# This should generate an error.
v11: TD11 = TD12(key="", value="")


v12 = [v10]
# This should generate an error.
v13: list[TD10] = v12

v14 = [v11]
# This should generate an error.
v15: list[TD12] = v14


class TD20(TypedDict):
    key: str
    value: str


class TD21(TypedDict):
    key: str
    value: str
    extra: str


# This should generate an error.
v20: TD21 = TD20(key="", value="")

v21: TD20 = TD21(key="", value="", extra="")


v22 = [v20]
# This should generate an error.
v23: list[TD20] = v22

v24: list[TD20] = [v21]
# This should generate an error.
v25: list[TD21] = v24



# This sample tests the case where a generic class constructor doesn't
# allow for the solving of a class-scoped type variable. In this case,
# the type argument should be Unknown.

from typing import Generic, TypeVar

T = TypeVar("T")


class A(Generic[T]):
    def __new__(cls, *args, **kwargs):
        return super().__new__(cls, *args, **kwargs)


class B(Generic[T]):
    def __init__(self): ...


class C(Generic[T]):
    def __new__(cls, *args, **kwargs):
        return super().__new__(cls, *args, **kwargs)

    def __init__(self): ...


class D(Generic[T]):
    def __new__(cls, *args, **kwargs):
        return super().__new__(cls, *args, **kwargs)

    def __init__(self, a: T): ...


class E(Generic[T]):
    pass


a = A(1)
reveal_type(a, expected_text="A[Unknown]")

b = B()
reveal_type(b, expected_text="B[Unknown]")

c = C()
reveal_type(c, expected_text="C[Unknown]")

d = D(1)
reveal_type(d, expected_text="D[int]")

e = E()
reveal_type(e, expected_text="E[Unknown]")



# This sample tests the use of ParamSpec along with Concatenate in
# a return type.

from typing import Callable, Protocol, TypeVar, Concatenate, ParamSpec
from threading import RLock
import functools


class HasLock(Protocol):
    _lock: RLock


S = TypeVar("S", bound=HasLock)
P = ParamSpec("P")
R = TypeVar("R")


def with_lock(func: Callable[Concatenate[S, P], R]) -> Callable[Concatenate[S, P], R]:
    @functools.wraps(func)
    def wrapper(self: S, *args: P.args, **kwargs: P.kwargs) -> R:
        with self._lock:
            return func(self, *args, **kwargs)

    return wrapper


class MyClass:
    def __init__(self):
        self._lock = RLock()

    @with_lock
    def test_1(self, param1: int) -> str: ...

    @with_lock
    def test_2(self) -> str: ...


@with_lock
def test_3(cls: MyClass, param1: int) -> str: ...


testClass = MyClass()

res1 = testClass.test_1(42)
reveal_type(res1, expected_text="str")

res2 = testClass.test_2()
reveal_type(res2, expected_text="str")

res3 = test_3(testClass, 42)
reveal_type(res3, expected_text="str")

res4: Callable[[MyClass, int], str] = with_lock(test_3)
reveal_type(res4, expected_text="(MyClass, int) -> str")



# This sample tests the analyzer's ability to handle inherited
# data classes.

# pyright: reportIncompatibleVariableOverride=false

from dataclasses import dataclass, field


class C1: ...


class C2: ...


class C3: ...


@dataclass
class DC1:
    aa: C1
    bb: C2
    cc: C3


class NonDC2:
    ff: int


@dataclass
class DC2(NonDC2, DC1):
    ee: C2
    aa: C2
    dd: C2


dc2_1 = DC2(C2(), C2(), C3(), C2(), C2())

# This should generate an error because the type
# of parameter aa has been replaced with type C1.
dc2_2 = DC2(C1(), C2(), C3(), C2(), C2())

dc2_3 = DC2(ee=C2(), dd=C2(), aa=C2(), bb=C2(), cc=C3())


@dataclass
class DC3:
    aa: C1
    bb: C2 = C2()
    cc: C3 = C3()


@dataclass
class DC4(DC3):
    # This should generate an error because
    # previous parameters have default values.
    dd: C1


@dataclass
class DC5(DC3):
    # This should not generate an error because
    # aa replaces aa in DC3, and it's ordered
    # before the params with default values.
    aa: C2


@dataclass
class DC6:
    a: int = 0


@dataclass
class DC7(DC6):
    # This should generate an error because it is overriding
    # a field with a default value, but it doesn't have a
    # default value.
    a: int

    # This should generate an error because the default
    # value for "a" is inherited from the base class.
    b: str


@dataclass
class DC8:
    a: int = field(default=0)


@dataclass
class DC9(DC8):
    # This should generate an error because it is overriding
    # a field with a default value, but it doesn't have a
    # default value.
    a: int

    # This should generate an error because the default
    # value for "a" is inherited from the base class.
    b: str


@dataclass
class DC10:
    a: str = field(init=False, default="s")
    b: bool = field()


@dataclass
class DC11(DC10):
    a: str = field()
    b: bool = field()



from typing import ClassVar, Final, Unpack, override


foo: Final = int()
bar: str = ""
baz: Final[str] = ""

class Foo:
    a: ClassVar = "asdf"
    b: ClassVar[str] = "asdf"

_ = list([1])

class Bar[T]:
    def __init__(self, value: T) -> None:
        self.value = value

    def foo(self) -> None: ...

_ = Bar(True)

_ = tuple((1,2,3))

class Baz[U, *T]:
    def __init__(self, asdf: U,*value: Unpack[T]) -> None:
        pass

_ = Baz([1], 1,2,"")

class Qux(Bar[int]):
    @override
    def foo(self) -> None:
        return super().foo()

qux: list[tuple[int, str]] = list()

class Quxx:
    def __init__(self, value) -> None: ...


_ = Quxx(value=1)


# This sample tests the reportUnusedImport diagnostic rule.
import sys as sys  # Assumes export
import os as os2  # Should error
from sys import path as p  # Should error
from os import environ as environ  # Assumes export



# This sample tests the handling of generic classes that are parameterized
# using a ParamSpec.

from typing import Callable, Generic, TypeVar, ParamSpec


_P = ParamSpec("_P")
_R = TypeVar("_R")


class MyDecorator(Generic[_P, _R]):
    def __init__(self, function: Callable[_P, _R]):
        self.function = function

    def __call__(self, *args: _P.args, **kwargs: _P.kwargs) -> _R:
        print("Inside Function Call")
        return self.function(*args, **kwargs)

    def do_stuff(self, name: str, *args: _P.args, **kwargs: _P.kwargs) -> int:
        return 0


@MyDecorator
def func1(x: int, y: int, *, z: int):
    return x + y


func1(6, 6, z=6)


@MyDecorator
def func2(*, a: int):
    pass


func2.do_stuff("hi", a=4)



# This sample tests that a generic callable passed to a function cannot
# be called with parameters that don't match the generic.

from typing import TypeVar, Callable, Any

T = TypeVar("T")


def func1(cb: Callable[[T], list[T]], param1: T, param2: Any, param3: Any | T) -> None:
    cb(param1)
    cb(param2)
    cb(param3)

    # This should generate an error.
    cb(0)



# This sample tests the logic that validates the second parameter to
# an isinstance or issubclass call and ensures that it's a class or
# tuple of classes.


import sys
from abc import abstractmethod
from typing import (
    Annotated,
    Any,
    Callable,
    Generic,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    TypedDict,
    Union,
)

if sys.version_info >= (3, 10):
    from types import NoneType
else:
    NoneType = type(None)

_T = TypeVar("_T", int, str)


class A(Generic[_T]):
    pass


a = A()

if isinstance(a, A):
    pass

# This should generate an error because generic types with
# subscripts are not allowed.
if isinstance(a, A[str]):
    pass

# This should generate an error in Python 3.9 and older because
# unions are not allowed, but this error isn't currently caught.
if issubclass(A, Union[A, int]):
    pass

if issubclass(A, type(None)):
    pass

if issubclass(A, NoneType):
    pass


class ClassA(Generic[_T]):
    v1: _T
    v2: Type[_T]

    @property
    @abstractmethod
    def _elem_type_(self) -> Union[Type[_T], Tuple[Type[_T], ...]]:
        raise NotImplementedError

    def check_type(self, var: Any) -> bool:
        return isinstance(var, self._elem_type_)

    def execute(self, var: Union[_T, Tuple[_T]]) -> None:
        if isinstance(var, self._elem_type_):
            pass

        if isinstance(var, type(self.v1)):
            pass

        if isinstance(var, self.v2):
            pass


def func1(exceptions: Sequence[type[BaseException]], exception: Exception):
    return isinstance(exception, tuple(exceptions))


if isinstance(a, Callable):
    ...

# This should generate an error because a subscripted Callable
# will result in a runtime exception.
if isinstance(a, Callable[[], Any]):
    ...

if isinstance(a, type(len)):
    ...


class TD1(TypedDict):
    a: int


# This should generate an error because TypedDict classes can't
# be used in an isinstance call.
if isinstance(a, TD1):
    pass


TA1 = Annotated[int, ""]

# This should generate two errors because Annotated can't be used
# in an isinstance call.
if isinstance(1, TA1):
    pass

# This should generate an error because Any can't be used
# in an isinstance call.
if isinstance(1, Any):
    pass

# This should generate an error because Literal can't be used
# in an isinstance call.
if isinstance(1, Literal[1, 2]):
    pass



# This sample tests that the type checker properly handles
# types of args and kwargs correctly.

from typing import Any, Hashable, Mapping, Protocol


def requires_hashable_tuple(p1: tuple[Hashable, ...]): ...


def requires_hashable_dict(p1: dict[str, Hashable]): ...


def test_args(*args: Hashable):
    if args:
        aaa = list(args)
        bbb = tuple(aaa)
        args = bbb
    requires_hashable_tuple(args)


def test_kwargs(**kwargs: Hashable):
    requires_hashable_dict(kwargs)


class StrSubclass(str): ...


def test_kwargs2(
    a: Mapping[str, Any],
    b: Mapping[Any, Hashable],
    c: dict[StrSubclass, Hashable],
    d: int,
    e: Mapping[int, Hashable],
    f: tuple[str, ...],
):
    test_kwargs(**a)
    test_kwargs(**b)
    test_kwargs(**c)

    # This should generate an error
    test_kwargs(**d)

    # This should generate an error
    test_kwargs(**e)

    # This should generate an error
    test_kwargs(**f)


class Callback1(Protocol):
    def __call__(self) -> None: ...


def func1(
    value: str = ...,
    *args: object,
) -> None: ...


def func2(
    value: str = ...,
    **kwargs: object,
) -> None: ...


def func3(
    value: str = ...,
    *args: object,
    **kwargs: object,
) -> None: ...


v1: Callback1 = func1
v2: Callback1 = func2
v3: Callback1 = func3



# This sample tests various arithmetic expressions.


def returnsInt1() -> int:
    a = 1
    b = 2

    return a + b % b // a - b // a


def returnsInt2() -> int:
    a = 1.0
    b = 2

    # This should generate an error because
    # it should evaluate to a float, which is
    # not compatible with the specified return
    # type.
    return a + b % b // a - b // a


def returnsFloat1() -> float:
    a = 1
    b = 2
    return a + b % b / a - b // a


def returnsFloat2() -> float:
    a = complex(1, 2)
    b = 2

    # This should generate an error because it
    # should evaluate to a complex, which is
    # not compatible with the specified return
    # type.
    return a + b % b / a - b // a


def returnsComplex1() -> complex:
    a = complex(1, 2)
    b = 2
    c = 4.0

    # This should generate an error because a
    # float should be divisible by a complex.
    return a + b % (b / a - c // a)


a = 3
b = 4

# This should generate an error because matrix multiply
# isn't supported for int.
c = a @ b



# This sample tests the handling of super() with no parameters
# and a base class with an annotated cls or self parameter that
# relies on the subclass being passed as a parameter.

from typing import Generic, TypeVar

_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2", bound="Parent2")


class Parent1(Generic[_T1]):
    @classmethod
    def construct(cls: type[_T1]) -> _T1:
        return cls()


class Child1(Parent1["Child1"]):
    @classmethod
    def construct(cls) -> "Child1":
        return super().construct()


reveal_type(Child1.construct(), expected_text="Child1")


class Parent2:
    @classmethod
    def construct(cls: type[_T2]) -> _T2: ...


class Child2(Parent2):
    @classmethod
    def construct(cls: type[_T2]) -> _T2:
        return super().construct()


reveal_type(Child2.construct(), expected_text="Child2")



# This sample tests that an overloaded method in a stub honors the
# @final on the first overload.

from typing import final, overload

class ClassA:
    @overload
    @final
    def method1(self, x: int) -> int: ...
    @overload
    def method1(self, x: str) -> str: ...
    @overload
    def method2(self, x: int) -> int: ...
    @overload
    @final
    # This should generate an error because the first overload
    # is not marked @final but this one is.
    def method2(self, x: str) -> str: ...

class ClassB(ClassA):
    @overload
    def method1(self, x: int) -> int: ...
    @overload
    # This should generate an error.
    def method1(self, x: str) -> str: ...



# This sample tests the case where a "bare" Final is used in a dataclass
# with a default value.

from typing import Final
from dataclasses import dataclass


@dataclass
class DC1:
    a: Final = 1


v1 = DC1(1)
reveal_type(v1.a, expected_text="Literal[1]")

v2 = DC1()
reveal_type(v2.a, expected_text="Literal[1]")



# This sample tests error conditions related to the use of PEP 695
# type parameters outside of their valid scope.


class ClassA[S]:
    s: S

    class ClassB[T](dict[S, T]):
        s: S
        t: T

        def method1[U](self):
            s: S
            t: T
            u: U
            lambda: (S, T, U)

    # This should generate an error because T is out of scope.
    t: T


# This should generate an error because S is out of scope.
s: S

# This should generate an error because T is out of scope.
t: T


def func1[A]():
    def func2[B]():
        a: A
        b: B

        class ClassC[C](dict[B, C]):
            a: A
            b: B
            c: C

            def method1[D](self):
                a: A
                b: B
                c: C
                d: D
                e = lambda: (A, B, C, D)

    a: A

    # This should generate an error because B is out of scope.
    b: B


# This should generate an error because A is out of scope.
a: A

# This should generate an error because B is out of scope.
b: B

type TA1[A] = list[A]

# This should generate an error because B is out of scope.
type TA2[A] = list[B]


S = 0


def outer1[S]():
    S = ""
    T = 1

    def outer2[T]():
        def inner1():
            nonlocal S  # OK
            reveal_type(S, expected_text="Literal['']")

        def inner2():
            global S  # OK
            reveal_type(S, expected_text="Literal[0]")


T = 0


class Outer2[T]:
    T = 1

    reveal_type(T, expected_text="Literal[1]")

    class Inner1:
        T = ""

        reveal_type(T, expected_text="Literal['']")

        def inner_method(self):
            reveal_type(T, expected_text="TypeVar")

    def outer_method(self):
        T = 3j

        reveal_type(T, expected_text="complex")

        def inner_func():
            reveal_type(T, expected_text="complex")


class Outer3[T]:
    # This should generate an error because Outer3 is
    # not bound at this point.
    def inner_func1[S](self: Outer3[S]): ...



# This sample tests the case where a callback protocol uses position-only
# parameters.

from typing import Any, Protocol


class P0(Protocol):
    def __call__(self, x: int, /, y: str) -> Any: ...


def test1(x: int, /, y: str, z: None = None) -> Any: ...


x: P0 = test1


class P1(Protocol):
    def __call__(self, *args: *tuple[int, int]): ...


class P2(Protocol):
    def __call__(self, x: int, y: int, /): ...


class P3(Protocol):
    def __call__(self, x: int, /, *args: *tuple[int]): ...


class P4(Protocol):
    def __call__(self, x: int, y: int = 2, /): ...


def test2(p1: P1, p2: P2, p3: P3, p4: P4):
    x1: P1 = p2
    x2: P1 = p3
    x3: P1 = p4



# This sample tests the case where an operator overload method is
# defined as a callable protocol object.

from typing import Protocol


class ComparisonOp(Protocol):
    def __call__(self, other: object, /) -> bool: ...


class Number:
    __lt__: ComparisonOp
    __le__: ComparisonOp
    __gt__: ComparisonOp
    __ge__: ComparisonOp


n1 = Number()
n2 = Number()

v1 = n1 < n2
v2 = n1 >= n2
v2 = n1 > n2



# This sample tests the constraint solver when a callable type is involved.

# pyright: strict

from typing import Callable, Literal, TypeVar


def filter_fn(value: object): ...


v1 = filter(filter_fn, [1, 2, 3])
reveal_type(v1, expected_text="filter[int]")

v2 = filter(filter_fn, {1, 2})
reveal_type(v2, expected_text="filter[int]")

v3 = filter(filter_fn, {1: 2})
reveal_type(v3, expected_text="filter[int]")


_T = TypeVar("_T")
Animal = Literal["cat"]


def func(v: Callable[[], _T]) -> _T: ...


x1: dict[Animal, int] = func(lambda: {"cat": 0})


def func1(factory: Callable[[], _T]) -> _T: ...


x2: set[int] = func1(lambda: set())



from dataclasses import dataclass
from typing import dataclass_transform

@dataclass_transform(
    skip_replace=True,
    frozen_default=True,
)
def frozen[T: type](t: T) -> T:
    return dataclass(frozen=True, slots=True)(t)




# This sample tests the case where a match statement is used in a loop
# and the subject is potentially narrowed in the loop, therefore creating
# a circular dependency.

from typing import Literal


def func1(lit: Literal["a", "b"]) -> None:
    for _ in range(2):
        match lit:
            case "a":
                v = "123"

            case "b":
                v = "234"

        v.replace(",", ".")



# This sample tests the PEP 695 type parameter syntax for generic classes
# and functions.

T1 = 0


class ClassA[T1]: ...


def func1[T1](): ...


T2: str


class ClassB[T2]: ...


def func2[T2](): ...


# This should generate an error because T3 is duplicated.
class ClassC[T3, S1, T3]: ...


class ClassD:
    class ClassE: ...

    class ClassF:
        class A[T]: ...

        int_alias = int

        class B(A[int_alias]):
            pass

        # This should generate an error because ClassE is out of scope.
        class C(A[ClassE]):
            pass


class ClassG[T](list["T"]):
    pass


class ClassH:
    def object[T](self, target: object, new: T) -> T: ...


# This should generate an error because T3 is duplicated.
def func3[T3, S1, T3](): ...


def func4[T4](T4: int): ...


def func5[T5](a: int):
    # This should generate an error because T5 is already in use.
    class ClassA[T5]: ...

    # This should generate an error because T5 is already in use.
    def inner_func1[T5](): ...


def func6[T6](T7: int):
    class ClassA[T7]: ...

    def inner_func1[T7](): ...

    global T2

    class ClassB[T2]:
        global T2

    class ClassC[T3]:
        T3 = 4

    T3 = 4


def func7[T8: ForwardRefClass[str], T9: "ForwardRefClass[int]"]():
    pass


def func8[T10: (ForwardRefClass[str], "ForwardRefClass[int]")]():
    pass


class ForwardRefClass[T]:
    pass


class ClassI1: ...


class ClassI2:
    def method1[T](self, v: ClassI1) -> None: ...

    # This should generate an error because ClassJ is
    def method2[T](self, v: ClassI3) -> None: ...


class ClassI3: ...


def func9[T, **P, S](x: T) -> T:
    S = 1

    def inner():
        # This should generate two errors.
        nonlocal T, P

        nonlocal S

    return x



# This sample tests the case where a invariant type parameter is used
# within a contravariant type argument.

from typing import TypeVar, Generic

T = TypeVar("T")
T_contra = TypeVar("T_contra", contravariant=True)


class Contra(Generic[T_contra]): ...


class Foo(Generic[T]): ...


class Bar(Foo[T]): ...


def func(x: Contra[Foo[int]]):
    v: Contra[Bar[int]] = x



# This sample tests the parsing and analysis of f-strings with empty {},
# with a backslash in the format specifier, and with a colon in a format
# specifier expression.

msg = "test"
a = f"{}"

message = "hi"
f"{message:\u3000>10}"


x = 0
precision = 3
print(f"{x: .{precision:d}f}")



import contextlib
from typing import Literal

def _(cm: contextlib.AbstractContextManager[object, Literal[True]]):
    with cm:
        raise Exception   
    print(1)  # reachable
 
def _(cm: contextlib.AbstractContextManager[object, Literal[False]]):
    with cm:
        raise Exception
    print(1)  # error: unreachable



# This sample tests the case where a generic class is parameterized
# by a ParamSpec, and this ParamSpec is used in a method with
# *args and **kwargs parameters. In cases where the ParamSpec captures
# a generic function, the TypeVars for this generic function should
# still be solvable.

from typing import Callable, Generic, ParamSpec, TypeVar


S = TypeVar("S")
T = TypeVar("T")

P = ParamSpec("P")
R = TypeVar("R")


def func1(a: S, b: T) -> dict[S, T]: ...


class DecoratorClass1(Generic[P, R]):
    def __init__(self, func: Callable[P, R]):
        self._func = func

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
        return self._func(*args, **kwargs)

    def other(self, val: int, *args: P.args, **kwargs: P.kwargs) -> R: ...


decorated_func1 = DecoratorClass1(func1)

reveal_type(
    decorated_func1,
    expected_text="DecoratorClass1[(a: S@func1, b: T@func1), dict[S@func1, T@func1]]",
)

func1_ret = decorated_func1(1, "")
reveal_type(func1_ret, expected_text="dict[int, str]")


func1_other_ret = decorated_func1.other(0, 1, "")
reveal_type(func1_other_ret, expected_text="dict[int, str]")


def func2(func: Callable[P, R]) -> Callable[P, R]: ...


d1 = func2(func1)
d2 = func2(d1)
d3 = d2(1, "")
reveal_type(d3, expected_text="dict[int, str]")



# This sample tests the case where a constrained TypeVar is used
# as an argument for a constructor or function call.

from typing import AnyStr, Iterable, TypeVar, Generic
from dataclasses import dataclass


class ClassA: ...


_T = TypeVar("_T", int | float, ClassA)


@dataclass
class Data(Generic[_T]):
    data: _T


def func1(a: Data[_T]) -> _T:
    if isinstance(a.data, (int, float)):
        value = int(a.data / 3)
        reveal_type(value, expected_text="int*")
    else:
        value = a.data
        reveal_type(value, expected_text="ClassA*")

    return value


def func2(val: AnyStr, objs: Iterable[AnyStr]) -> AnyStr:
    return val.join(objs)



# This sample tests that __future__ imports are found
# only at the beginning of a file.

"""Doc String"""

"Extension"
from __future__ import annotations  # This should generate an error


def func():
    from __future__ import annotations  # This should generate an error



# This sample tests a loop where there are multiple symbols
# that depend on each other.

# pyright: strict


def func1() -> str | None: ...


s1: str | None = None
s2 = None
while True:
    obj = func1()

    x = s2
    condition = obj and obj != s1

    s1 = obj
    s2 = obj



# This sample tests support for Concatenate with a ... type argument.

from typing import Callable, Concatenate

TA1 = Callable[Concatenate[int, ...], None]


def func1(cb: Callable[Concatenate[int, str, ...], None]): ...


def func2(cb: TA1): ...


def cb1(x: int, y: str, z: str) -> None: ...


func1(cb1)
func2(cb1)


def cb2(x: int, y: str, *args: int, **kwargs: str) -> None: ...


func1(cb2)
func2(cb2)


def cb3(x: str, y: str) -> None: ...


# This should generate an error.
func1(cb3)

# This should generate an error.
func2(cb3)



# This sample tests the case where a constructor for a class is invoked
# multiple times as separate arguments for a call.

# pyright: strict

from __future__ import annotations
from typing import Any, Callable, Generic, Iterable, TypeVar, overload

T = TypeVar("T")
S = TypeVar("S", covariant=True)


class ParentA: ...


class ChildA(ParentA, Generic[T]):
    def __init__(self, a: T) -> None: ...


def func1(arg1: ParentA, arg2: ParentA): ...


func1(ChildA(1), ChildA(2))


class ParentB(Generic[T]): ...


class ChildB(ParentB[T]):
    def __init__(self, a: T) -> None: ...


def func2(arg1: ParentB[T], arg2: ParentB[T]) -> T: ...


# This should generate an error.
func2(ChildB(""), ChildB(1.2))


class ClassC(Generic[S]):
    def __new__(cls, item: S) -> "ClassC[S]": ...

    def __call__(self, obj: Any) -> S: ...


def func3(func1: Callable[..., T], func2: Callable[..., T]) -> T: ...


x2 = func3(ClassC(""), ClassC(1))
reveal_type(x2, expected_text="str | int")


class ClassD(Generic[S]):
    @overload
    def __new__(cls, item: S, /) -> ClassD[S]: ...

    @overload
    def __new__(cls, item: S, __item2: S, /) -> ClassD[tuple[S, S]]: ...

    def __new__(cls, *items: Any) -> Any: ...

    def __call__(self, obj: Any) -> Any: ...


func3(ClassD(""), ClassD(""))


def func4(a: Iterable[tuple[str, ...]]):
    zip(a, zip(*a))



import sys
from collections.abc import Iterable, Mapping, Sequence
from types import GenericAlias
from typing import Any, AnyStr, Final, Generic, Literal, NamedTuple, Protocol, overload, type_check_only
from typing_extensions import TypeAlias

__all__ = [
    "urlparse",
    "urlunparse",
    "urljoin",
    "urldefrag",
    "urlsplit",
    "urlunsplit",
    "urlencode",
    "parse_qs",
    "parse_qsl",
    "quote",
    "quote_plus",
    "quote_from_bytes",
    "unquote",
    "unquote_plus",
    "unquote_to_bytes",
    "DefragResult",
    "ParseResult",
    "SplitResult",
    "DefragResultBytes",
    "ParseResultBytes",
    "SplitResultBytes",
]

uses_relative: Final[list[str]]
uses_netloc: Final[list[str]]
uses_params: Final[list[str]]
non_hierarchical: Final[list[str]]
uses_query: Final[list[str]]
uses_fragment: Final[list[str]]
scheme_chars: Final[str]
if sys.version_info < (3, 11):
    MAX_CACHE_SIZE: Final[int]

class _ResultMixinStr:
    __slots__ = ()
    def encode(self, encoding: str = "ascii", errors: str = "strict") -> _ResultMixinBytes: ...

class _ResultMixinBytes:
    __slots__ = ()
    def decode(self, encoding: str = "ascii", errors: str = "strict") -> _ResultMixinStr: ...

class _NetlocResultMixinBase(Generic[AnyStr]):
    __slots__ = ()
    @property
    def username(self) -> AnyStr | None: ...
    @property
    def password(self) -> AnyStr | None: ...
    @property
    def hostname(self) -> AnyStr | None: ...
    @property
    def port(self) -> int | None: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias:
        """
        Represent a PEP 585 generic type

        E.g. for t = list[int], t.__origin__ is list and t.__args__ is (int,).
        """
        ...

class _NetlocResultMixinStr(_NetlocResultMixinBase[str], _ResultMixinStr):
    __slots__ = ()

class _NetlocResultMixinBytes(_NetlocResultMixinBase[bytes], _ResultMixinBytes):
    __slots__ = ()

class _DefragResultBase(NamedTuple, Generic[AnyStr]):
    url: AnyStr
    fragment: AnyStr

class _SplitResultBase(NamedTuple, Generic[AnyStr]):
    scheme: AnyStr
    netloc: AnyStr
    path: AnyStr
    query: AnyStr
    fragment: AnyStr

class _ParseResultBase(NamedTuple, Generic[AnyStr]):
    scheme: AnyStr
    netloc: AnyStr
    path: AnyStr
    params: AnyStr
    query: AnyStr
    fragment: AnyStr

# Structured result objects for string data
class DefragResult(_DefragResultBase[str], _ResultMixinStr):
    def geturl(self) -> str: ...

class SplitResult(_SplitResultBase[str], _NetlocResultMixinStr):
    def geturl(self) -> str: ...

class ParseResult(_ParseResultBase[str], _NetlocResultMixinStr):
    def geturl(self) -> str: ...

# Structured result objects for bytes data
class DefragResultBytes(_DefragResultBase[bytes], _ResultMixinBytes):
    def geturl(self) -> bytes: ...

class SplitResultBytes(_SplitResultBase[bytes], _NetlocResultMixinBytes):
    def geturl(self) -> bytes: ...

class ParseResultBytes(_ParseResultBase[bytes], _NetlocResultMixinBytes):
    def geturl(self) -> bytes: ...

def parse_qs(
    qs: AnyStr | None,
    keep_blank_values: bool = False,
    strict_parsing: bool = False,
    encoding: str = "utf-8",
    errors: str = "replace",
    max_num_fields: int | None = None,
    separator: str = "&",
) -> dict[AnyStr, list[AnyStr]]: ...
def parse_qsl(
    qs: AnyStr | None,
    keep_blank_values: bool = False,
    strict_parsing: bool = False,
    encoding: str = "utf-8",
    errors: str = "replace",
    max_num_fields: int | None = None,
    separator: str = "&",
) -> list[tuple[AnyStr, AnyStr]]: ...
@overload
def quote(string: str, safe: str | Iterable[int] = "/", encoding: str | None = None, errors: str | None = None) -> str: ...
@overload
def quote(string: bytes | bytearray, safe: str | Iterable[int] = "/") -> str: ...
def quote_from_bytes(bs: bytes | bytearray, safe: str | Iterable[int] = "/") -> str: ...
@overload
def quote_plus(string: str, safe: str | Iterable[int] = "", encoding: str | None = None, errors: str | None = None) -> str: ...
@overload
def quote_plus(string: bytes | bytearray, safe: str | Iterable[int] = "") -> str: ...
def unquote(string: str | bytes, encoding: str = "utf-8", errors: str = "replace") -> str: ...
def unquote_to_bytes(string: str | bytes | bytearray) -> bytes: ...
def unquote_plus(string: str, encoding: str = "utf-8", errors: str = "replace") -> str: ...
@overload
def urldefrag(url: str) -> DefragResult: ...
@overload
def urldefrag(url: bytes | bytearray | None) -> DefragResultBytes: ...

# The values are passed through `str()` (unless they are bytes), so anything is valid.
_QueryType: TypeAlias = (
    Mapping[str, object]
    | Mapping[bytes, object]
    | Mapping[str | bytes, object]
    | Mapping[str, Sequence[object]]
    | Mapping[bytes, Sequence[object]]
    | Mapping[str | bytes, Sequence[object]]
    | Sequence[tuple[str | bytes, object]]
    | Sequence[tuple[str | bytes, Sequence[object]]]
)

@type_check_only
class _QuoteVia(Protocol):
    @overload
    def __call__(self, string: str, safe: str | bytes, encoding: str, errors: str, /) -> str: ...
    @overload
    def __call__(self, string: bytes, safe: str | bytes, /) -> str: ...

def urlencode(
    query: _QueryType,
    doseq: bool = False,
    safe: str | bytes = "",
    encoding: str | None = None,
    errors: str | None = None,
    quote_via: _QuoteVia = ...,
) -> str: ...
def urljoin(base: AnyStr, url: AnyStr | None, allow_fragments: bool = True) -> AnyStr: ...
@overload
def urlparse(url: str, scheme: str = "", allow_fragments: bool = True) -> ParseResult: ...
@overload
def urlparse(
    url: bytes | bytearray | None, scheme: bytes | bytearray | None | Literal[""] = "", allow_fragments: bool = True
) -> ParseResultBytes: ...
@overload
def urlsplit(url: str, scheme: str = "", allow_fragments: bool = True) -> SplitResult:
    """
    Parse a URL into 5 components:
    <scheme>://<netloc>/<path>?<query>#<fragment>

    The result is a named 5-tuple with fields corresponding to the
    above. It is either a SplitResult or SplitResultBytes object,
    depending on the type of the url parameter.

    The username, password, hostname, and port sub-components of netloc
    can also be accessed as attributes of the returned object.

    The scheme argument provides the default value of the scheme
    component when no scheme is found in url.

    If allow_fragments is False, no attempt is made to separate the
    fragment component from the previous component, which can be either
    path or query.

    Note that % escapes are not expanded.
    """
    ...

if sys.version_info >= (3, 11):
    @overload
    def urlsplit(
        url: bytes | None, scheme: bytes | None | Literal[""] = "", allow_fragments: bool = True
    ) -> SplitResultBytes:
        """
        Parse a URL into 5 components:
        <scheme>://<netloc>/<path>?<query>#<fragment>

        The result is a named 5-tuple with fields corresponding to the
        above. It is either a SplitResult or SplitResultBytes object,
        depending on the type of the url parameter.

        The username, password, hostname, and port sub-components of netloc
        can also be accessed as attributes of the returned object.

        The scheme argument provides the default value of the scheme
        component when no scheme is found in url.

        If allow_fragments is False, no attempt is made to separate the
        fragment component from the previous component, which can be either
        path or query.

        Note that % escapes are not expanded.
        """
        ...

else:
    @overload
    def urlsplit(
        url: bytes | bytearray | None, scheme: bytes | bytearray | None | Literal[""] = "", allow_fragments: bool = True
    ) -> SplitResultBytes: ...

# Requires an iterable of length 6
@overload
def urlunparse(components: Iterable[None]) -> Literal[b""]: ...  # type: ignore[overload-overlap]
@overload
def urlunparse(components: Iterable[AnyStr | None]) -> AnyStr: ...

# Requires an iterable of length 5
@overload
def urlunsplit(components: Iterable[None]) -> Literal[b""]: ...  # type: ignore[overload-overlap]
@overload
def urlunsplit(components: Iterable[AnyStr | None]) -> AnyStr: ...
def unwrap(url: str) -> str: ...



# This sample tests a case involving nested protocol types that was
# not working in a previous version of the type checker.

from typing import Protocol


class Proto1[T](Protocol):
    x: T


class Proto2[T](Protocol):
    @staticmethod
    def a() -> Proto1[T]: ...
    @classmethod
    def b(cls) -> list[T]: ...


class C[T]:
    @staticmethod
    def a() -> T: ...
    @classmethod
    def b[S](cls: type[Proto2[S]]) -> list[S]: ...


def test[S](x: type[Proto2[S]]): ...


test(C[Proto1[int]])



# This sample tests the handling of __index__ magic method
# when used with the __getitem__ and __setitem__ method.


from typing import Generic, Literal, Self, Type, TypeVar, Any


class MyInt:
    def __init__(self, value: int) -> None:
        self.value = value

    def __index__(self) -> int:
        return self.value


l = ["foo", "bar"]
t = ("foo", "bar")

hex(MyInt(7))
l[MyInt(0)]
l[MyInt(0)] = "hi"
t[MyInt(1)]


class MyNonInt:
    def __init__(self) -> None:
        pass


# These should generate errors
hex(MyNonInt())
l[MyNonInt()]
l[MyNonInt()] = "hi"
t[MyNonInt()]


T = TypeVar("T")


class MyMetaclass(type):
    def __getitem__(cls: Type[T], item: int) -> T:
        return cls()


class ClassA(metaclass=MyMetaclass):
    pass


a1 = ClassA[1]
reveal_type(a1, expected_text="ClassA")

# This should generate an error
ClassA["1"]


class ClassB:
    def __setitem__(self, index: int, value: "ClassB"): ...


class ClassC:
    def __setitem__(self, index: int, value: "ClassC"): ...


B_or_C = TypeVar("B_or_C", ClassB, ClassC)


def func1(container: B_or_C):
    a = container
    a[1] = container


TD = TypeVar("TD", bound="ClassD[Any]")


class ClassD(Generic[TD]):
    def __setitem__(self, index: int, value: TD): ...


def func2(container: ClassD[TD], value: TD):
    container[1] = value


class ClassE:
    def __getattr__(self, s: str) -> Any:
        raise NotImplementedError()


e = ClassE()

# This should generate an error
v_e = e["test"]

# This should generate an error
e["test"] = 3


class ClassF(Generic[T]):
    def __getitem__(self, args: int) -> Self: ...

    def get(self, index: int) -> Self:
        reveal_type(self[index], expected_text="Self@ClassF[T@ClassF]")
        return self[index]


class ClassG:
    __slots__ = ["x"]


def func3(g: ClassG):
    reveal_type(g.x, expected_text="Unbound")
    reveal_type(g.x[0], expected_text="Unknown")


class ClassH:
    def __call__(self, *args, **kwargs) -> Self:
        return self


class ClassI:
    __getitem__ = ClassH()


reveal_type(ClassI()[0], expected_text="ClassH")


def func4(l: list[Literal["a", "b"]]):
    l[0] = "a"
    l[0:0] = ["a", "b"]



from __future__ import annotations

import sys
from pathlib import Path

from nodejs_wheel.executable import node


def run(script_name: str):
    sys.exit(node([str(Path(__file__).parent / f"{script_name}.js"), *sys.argv[1:]]))



# This sample tests various error conditions for the Self type

from typing import Callable, Generic, TypeVar
from typing_extensions import Self  # pyright: ignore[reportMissingModuleSource]


T = TypeVar("T")


# This should generate an error because Self can't be used in this context.
class A(Self): ...


# This should generate an error because Self can't be used in this context.
x: Self


def func1() -> None:
    # This should generate an error because Self can't be used in this context.
    x: Self


# This should generate an error because Self can't be used in this context.
def func2(a: Self) -> None: ...


# This should generate an error because Self can't be used in this context.
def func3() -> Self: ...


def is_self(t: object):
    return t is Self


class B:
    x: Self

    def method1(self) -> Self:
        return self

    def method2(self, a: Self) -> None:
        x: Self = a
        y = Self

    def method3(self: Self) -> Self:
        # This should generate an error because Self doesn't accept a type arg.
        y: Self[int]
        return self

    # This should generate an error because Self can't be used with
    # methods that declare a non-Self type for "self".
    def method4(self: T, a: Self) -> T:
        # This should generate an error because Self can't be used with
        # methods that declare a non-Self type for "self".
        x: Self

        return self

    @classmethod
    def method5(cls) -> type[Self]:
        return cls

    @classmethod
    def method6(cls, a: Self) -> None: ...

    @classmethod
    def method7(cls: type[Self]) -> type[Self]:
        return cls

    # This should generate an error because Self can't be used with
    # methods that declare a non-Self type for "self".
    @classmethod
    def method8(cls: type[T], a: Self) -> type[T]:
        # This should generate an error because Self can't be used with
        # methods that declare a non-Self type for "self".
        x: Self
        return cls

    # This should generate an error because Self can't be used in
    # a static method.
    @staticmethod
    def stat_method1(a: Self) -> None:
        # This should generate an error because Self can't be used in
        # a static method.
        x: Self


class C:
    @classmethod
    def outer(cls) -> Callable[[int, Self], Self]:
        def inner(_: int, bar: Self) -> Self:
            return bar

        return inner


class D(Generic[T]): ...


# This should generate an error because "Self" cannot be used
# within a generic class definition.
class E(D[Self]): ...


class MetaA(type):
    # This should generate an error because "Self" isn't
    # allowed in a metaclass.
    def __new__(cls, *args: object) -> Self: ...

    # This should generate an error because "Self" isn't
    # allowed in a metaclass.
    def __mul__(cls, count: int) -> list[Self]: ...



# This sample validates that a member's descriptor protocol is
# accessed via a member access expression only when accessing it
# through a class variable, not through an instance variable.

from typing import Any


class Descriptor:
    def __get__(self, obj: Any, objtype: Any = None) -> float:
        return 1.0


class ClassA:
    x: Descriptor

    def __init__(self, x: Descriptor):
        reveal_type(self.x, expected_type="float")

    def func1(self):
        reveal_type(self.x, expected_type="float")


class ClassB:
    def __init__(self, x: Descriptor):
        self.x = x
        reveal_type(self.x, expected_type="Descriptor")

    def func1(self):
        reveal_type(self.x, expected_type="Descriptor")



# This sample verifies that unknown types are properly eliminated from
# a loop.

# pyright: strict


def func1(a: int, b: str, c: str):
    v1: list[tuple[str, str, str]] = []
    for _ in range(0):
        if a == 0:
            print(v1[-1][0])
            last = v1[-1]
            v1[-1] = (b, last[1], c)



# This sample tests error cases for calls to the TypeAliasType constructor.

from typing import TypeVar
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    TypeAliasType,
)

# This should generate an error because arguments are missing.
TA1 = TypeAliasType()

# This should generate two errors because 1 isn't a legal name or str.
TA2 = TypeAliasType(1, int)

my_str = ""

# This should generate an error because my_str isn't a string literal.
TA3 = TypeAliasType(my_str, int)

# This should generate an error because name doesn't match.
TA4 = TypeAliasType("TA3", int)

# This should generate an error because it's not part of an assignment statement.
TypeAliasType("TA3", int)

# This should generate an error because it has an extra argument.
TA5 = TypeAliasType("TA5", int, x=3)

# This should generate an error because it has an extra argument.
TA6 = TypeAliasType("TA6", int, 3)

# This should generate two errors because type_params is not a tuple.
TA7 = TypeAliasType("TA7", int, type_params=[1])

# This should generate two errors because type_params is not a tuple of TypeVars.
TA8 = TypeAliasType("TA8", int, type_params=(int,))


S = TypeVar("S")
T = TypeVar("T")

# This should generate an error because S is not in scope.
TA9 = TypeAliasType("TA9", list[S], type_params=(T,))

my_tuple = (S, T)

# This should generate two errors because type_params is not a tuple expression.
TA10 = TypeAliasType("TA10", int, type_params=my_tuple)
print(TA10.__value__)


TA11 = TypeAliasType("TA11", int)
print(TA11.__value__)

type TA12 = int | str
print(TA12.__value__)

# This should generate an error.
TA13 = TypeAliasType("TA13", ...)


def func1():
    # This should generate an error.
    TA14 = TypeAliasType("TA14", int)



# This sample tests the handling of frozen dataclass types.

from dataclasses import dataclass
from typing import ClassVar


@dataclass(frozen=False)
class DC1:
    val1: int = 6


@dataclass(frozen=True)
class DC2:
    val2: float = 4


# This should generate an error because a frozen dataclass
# cannot inherit from a non-frozen dataclass.
@dataclass(frozen=True)
class DC3(DC1):
    val3: int = 4


@dataclass(frozen=True)
class DC4(DC2):
    val4: int = 4

    val5: ClassVar[int]


# This should generate an error because a non-frozen dataclass
# cannot inherit from a frozen dataclass.
@dataclass(frozen=False)
class DC5(DC2):
    val4: int = 5


a = DC1(val1=3)
a.val1 = 3

b = DC4(val2=3, val4=5)

DC4.val5 = 3

# This should generate an error because the dataclass is frozen.
b.val2 = 3

# This should generate an error because the dataclass is frozen.
b.val4 = 3


@dataclass(frozen=True)
class DC6(DC2):
    val2: int = 6



# This sample tests the type checker's handling of the NoReturn annotation type.

from typing import Callable, NoReturn, overload


# This should generate an error because the function
# implicitly returns None.
def func1() -> NoReturn:
    pass


def func2(x: bool) -> NoReturn:
    if x:
        # This should generate an error because the function
        # explicitly returns a value.
        return 4

    raise Exception()


def func3() -> NoReturn:
    raise Exception()


def func4(x: bool) -> str:
    if x:
        return "hello"
    else:
        func3()


def func5(x: bool) -> NoReturn:
    if x:
        # This should generate an error because the function
        # explicitly yields a value.
        yield 4

    raise Exception()


x1: Callable[[bool], bool] = func2


async def func6() -> NoReturn: ...


async def func7() -> NoReturn:
    await func6()


class A:
    def __new__(cls) -> NoReturn: ...


def func8() -> NoReturn:
    A()


class C:
    def __call__(self) -> NoReturn: ...


def func10() -> NoReturn:
    C()()


@overload
def func11() -> NoReturn: ...


@overload
def func11(x: int) -> None: ...


def func11(x: int = 0) -> NoReturn | None: ...


def func12() -> NoReturn:
    func11()


def func13() -> NoReturn:
    # This should generate an error.
    func11(0)


def func14(x: int) -> NoReturn: ...


def func15():
    # This should generate an error.
    return func14()



# This sample tests various conditions under which variadic
# type variables can and cannot be used.

# pyright: reportMissingModuleSource=false

from typing import Generic, TypeVar, Union
from typing_extensions import TypeVarTuple, Unpack


_T = TypeVar("_T")
_Xs = TypeVarTuple("_Xs")


class ClassA(Generic[_T, Unpack[_Xs]]):
    def __init__(self, *args: Unpack[_Xs]) -> None:
        reveal_type(args, expected_text="tuple[*_Xs@ClassA]")

    # This should generate two errors.
    def func2(self) -> Union[_Xs]: ...

    def func3(self) -> tuple[Unpack[_Xs]]: ...

    # This should generate an error.
    def func4(self) -> tuple[_Xs]: ...

    def func5(self) -> "ClassA[int, str, Unpack[_Xs]]": ...

    # This should be an error because list doesn't accept a variadic TypeVar.
    x: list[_Xs] = []

    # This should generate an error.
    y: _Xs = ()

    # This should generate an error.
    z: tuple[_Xs, ...]


# This should generate an error.
class ClassB(Generic[_Xs]): ...


# This should generate an error.
x: list[_Xs] = []

# This should generate an error.
y: _Xs = ()


# This should generate an error because of the name mismatch.
BadName = TypeVarTuple("Ts1")

# This should generate TypeVarTuple cannot have constraints.
Ts2 = TypeVarTuple("Ts2", int, str)

# This should generate TypeVarTuple cannot be covariant.
Ts3 = TypeVarTuple("Ts3", covariant=True)

# This should generate TypeVarTuple cannot be contravariant.
Ts4 = TypeVarTuple("Ts4", contravariant=True)

# This should generate TypeVarTuple does not accept other keyword arguments.
Ts5 = TypeVarTuple("Ts5", other=True)



# This sample tests the type: ignore for individual lines.
# It uses a form of ignore syntax that is not part of the
# official PEP 484 spec but is a variant supported by mypy.

from typing import Dict


a: int = 3
b = len(a) # type: ignore[1424]

for for for # type: ignore[1424, 244]

c: Dict[str, str] = {
    3: 3,
    'hello': 3,
    3.2: 2.4
} #type:ignore[999] # something








# This sample tests the usage of overloads in decorators.

from typing import Callable, TypeVar, overload, Optional, Union

F = TypeVar("F", bound=Callable[[], None])


@overload
def atomic(__func: F) -> F: ...


@overload
def atomic(*, savepoint: bool = True) -> Callable[[F], F]: ...


def atomic(
    __func: Optional[Callable[..., None]] = None, *, savepoint: bool = True
) -> Union[Callable[[], None], Callable[[F], F]]: ...


@atomic
def func1() -> None: ...


@atomic(savepoint=False)
def func2() -> None: ...



# This sample tests the case where a dataclass field has a corresponding
# redundant declaration within a method.

from dataclasses import dataclass


@dataclass
class ClassA:
    a: int
    b: str

    def foo(self):
        self.b: str = ""


ClassA(1, "hi")



# This sample tests the case where a comprehension requires bidirectional
# type inference for correct analysis.

# pyright: strict

from typing import TypedDict


class X(TypedDict):
    x: str


xs: list[X] = []
xs.extend({"x": c} for c in "abc")


def func1(data: dict[str, int]):
    sum(data.get(k, 0) for k in "")


def func2(data: dict[str, int]):
    sum([data.get(k, 0) for k in ""])



# This sample tests the case where a lambda is assigned to
# a union type that contains multiple callables.

from typing import Callable, Protocol, TypeVar


U1 = Callable[[int, str], bool] | Callable[[str], bool]


def accepts_u1(cb: U1) -> U1:
    return cb


def callback_1(p0: int, p1: str):
    return True


def callback_2(p0: str):
    return True


def callback_3(*p0: str):
    return True


accepts_u1(lambda s: s.startswith("hello"))
accepts_u1(lambda i, s: i > 0 and s.startswith("hello"))
accepts_u1(lambda *i: True)
accepts_u1(callback_1)
accepts_u1(callback_2)
accepts_u1(callback_3)

# This should generate an error
accepts_u1(lambda a, b, c: True)


class Callable1(Protocol):
    def __call__(self, p0: int, p1: str) -> bool: ...


class Callable2(Protocol):
    def __call__(self, p0: str) -> bool: ...


class Callable3(Protocol):
    def __call__(self, *p0: str) -> bool: ...


class Callable4(Protocol):
    def __call__(self, p0: int, p1: str, *p2: str) -> bool: ...


U2 = Callable1 | Callable2 | Callable3 | Callable4


def accepts_u2(cb: U2) -> U2:
    return cb


accepts_u2(lambda p0: p0.startswith("hello"))
accepts_u2(lambda p0, p1: p0 > 0 and p1.startswith("hello"))
accepts_u2(lambda *i: True)
accepts_u2(lambda p0, p1, *p2: True)
accepts_u2(callback_1)
accepts_u2(callback_2)
accepts_u2(callback_3)


T = TypeVar("T")

Takes = Callable[[T], object]

U3 = Takes[Takes[int]] | Takes[Takes[str]]


def accepts_u3(u: U3):
    # This should generate an error.
    u(lambda v: v.lower())



# This sample tests the case where a constructor is converted to
# a callable.


from typing import (
    Any,
    Callable,
    Generic,
    NoReturn,
    ParamSpec,
    Self,
    TypeVar,
    overload,
    reveal_type,
)

P = ParamSpec("P")
R = TypeVar("R")
T = TypeVar("T")


def accepts_callable(cb: Callable[P, R]) -> Callable[P, R]:
    return cb


class Class1:
    def __init__(self, x: int) -> None:
        pass


r1 = accepts_callable(Class1)
reveal_type(r1, expected_text="(x: int) -> Class1")
reveal_type(r1(1), expected_text="Class1")


class Class2:
    pass


r2 = accepts_callable(Class2)
reveal_type(r2, expected_text="() -> Class2")
reveal_type(r2(), expected_text="Class2")


class Class3:
    def __new__(cls, *args, **kwargs) -> Self: ...

    def __init__(self, x: int) -> None: ...


r3 = accepts_callable(Class3)
reveal_type(r3, expected_text="(x: int) -> Class3")
reveal_type(r3(3), expected_text="Class3")


class Class4:
    def __new__(cls, x: int) -> int: ...


r4 = accepts_callable(Class4)
reveal_type(r4, expected_text="(x: int) -> int")
reveal_type(r4(1), expected_text="int")


class Meta1(type):
    def __call__(cls, *args: Any, **kwargs: Any) -> NoReturn:
        raise NotImplementedError("Class not constructable")


class Class5(metaclass=Meta1):
    def __new__(cls, *args: Any, **kwargs: Any) -> Self:
        return super().__new__(cls)


r5 = accepts_callable(Class5)
reveal_type(r5, expected_text="(...) -> NoReturn")


class Class6Proxy: ...


class Class6:
    def __new__(cls) -> Class6Proxy:
        # This should generate an error because "cls" isn't compatible.
        return Class6Proxy.__new__(cls)

    def __init__(self, x: int) -> None:
        pass


r6 = accepts_callable(Class6)
reveal_type(r6, expected_text="() -> Class6Proxy")
reveal_type(r6(), expected_text="Class6Proxy")


class Class6_2:
    def __new__(cls) -> Any:
        return super().__new__(cls)

    def __init__(self, x: int) -> None:
        pass


r6_2 = accepts_callable(Class6_2)
reveal_type(r6_2, expected_text="() -> Any")
reveal_type(r6_2(), expected_text="Any")


class Class7(Generic[T]):
    @overload
    def __init__(self: "Class7[int]", x: int) -> None: ...

    @overload
    def __init__(self: "Class7[str]", x: str) -> None: ...

    def __init__(self, x: int | str) -> None:
        pass


r7 = accepts_callable(Class7)
reveal_type(
    r7, expected_text="Overload[(x: int) -> Class7[int], (x: str) -> Class7[str]]"
)

reveal_type(r7(0), expected_text="Class7[int]")
reveal_type(r7(""), expected_text="Class7[str]")


class Class8(Generic[T]):
    def __new__(cls, x: T, y: list[T]) -> Self:
        return super().__new__(cls)


r8 = accepts_callable(Class8)
reveal_type(r8, expected_text="(x: T@Class8, y: list[T@Class8]) -> Class8[T@Class8]")
reveal_type(r8("", [""]), expected_text="Class8[str]")


class Class9:
    def __init__(self, x: list[T], y: list[T]) -> None:
        pass


r9 = accepts_callable(Class9)
reveal_type(r9, expected_text="(x: list[T@__init__], y: list[T@__init__]) -> Class9")
reveal_type(r9([""], [""]), expected_text="Class9")


M = TypeVar("M")


class Meta2(type):
    def __call__(cls: type[M], *args: Any, **kwargs: Any) -> M: ...


class Class10(metaclass=Meta2):
    def __new__(cls, x: int, y: str) -> Self:
        return super().__new__(cls)


r10 = accepts_callable(Class10)
reveal_type(r10, expected_text="(x: int, y: str) -> Class10")
reveal_type(r10(1, ""), expected_text="Class10")



# This sample tests that literal values are retained by the constraint
# solver if they are found as type arguments.

from typing import Literal


_L1 = Literal["foo", "bar"]


def combine(set1: set[_L1], set2: set[_L1]) -> None:
    x = set1 | set2
    reveal_type(x, expected_text="set[Literal['foo', 'bar']]")



# This sample tests ParamSpec processing when the source
# callable type has keyword-only or positional-only
# parameter separators.

from typing import Callable, ParamSpec, TypeVar


P = ParamSpec("P")
R = TypeVar("R")


def decorator(fn: Callable[P, R]) -> Callable[P, R]:
    return fn


def func1(*, value: str) -> None: ...


f1 = decorator(func1)
reveal_type(f1, expected_text="(*, value: str) -> None")


def func2(value: str, /) -> None: ...


f2 = decorator(func2)
reveal_type(f2, expected_text="(value: str, /) -> None")



# This sample tests that the NoReturn logic is able to handle
# union types in call expressions.

from typing import NoReturn


def func1() -> NoReturn:
    raise TypeError


class B:
    def always_noreturn(self) -> NoReturn:
        func1()

    def sometimes_noreturn(self) -> NoReturn:
        raise TypeError


class C:
    def always_noreturn(self) -> NoReturn:
        func1()

    def sometimes_noreturn(self) -> int:
        return 0


class A:
    def __init__(self):
        # Note the union type declaration here.
        self._B_or_C: B | C = B()

    def m3(self) -> NoReturn:
        self._B_or_C.always_noreturn()

    def m4(self) -> int:
        x = self._B_or_C.sometimes_noreturn()
        return x



# This sample tests the handling of generic type aliases that are
# defined in terms of other generic type aliases in a nested manner.

from typing import Awaitable, Callable, Generic, Literal, TypeAlias, TypeVar

TSource = TypeVar("TSource")
TError = TypeVar("TError")
TResult = TypeVar("TResult")
TNext = TypeVar("TNext")


class Context(Generic[TResult]):
    Response: TResult


class Result(Generic[TResult, TError]):
    def map(
        self, mapper: Callable[[Context[TResult]], TResult]
    ) -> "Result[TResult, TError]":
        return Result()


HttpFuncResult = Result[Context[TResult], TError]
HttpFuncResultAsync = Awaitable[Result[Context[TResult], TError]]

HttpFunc = Callable[
    [Context[TNext]],
    HttpFuncResultAsync[TResult, TError],
]

HttpHandler = Callable[
    [
        HttpFunc[TNext, TResult, TError],
        Context[TSource],
    ],
    HttpFuncResultAsync[TResult, TError],
]


async def run_async(
    ctx: Context[TSource],
    handler: HttpHandler[str, TResult, TError, TSource],
) -> Result[TResult, TError]:
    result = Result[TResult, TError]()

    def mapper(x: Context[TResult]) -> TResult:
        return x.Response

    return result.map(mapper)


T1 = TypeVar("T1", bound=Literal["a", "b", "c"])
T2 = TypeVar("T2", bound=Literal["b", "c"])

TA2: TypeAlias = list[T1]
TA3: TypeAlias = TA2[T2]



# This sample tests that an unbound variable that is generated in
# a function does not propagate beyond that function to callers.


def func1():
    # This should generate an error
    return a


# This should not.
b = func1()
reveal_type(b, expected_text="Unknown")


def func2(val: int):
    if val < 3:
        return val

    # This should generate an error
    return a


# This should not.
c = func2(36)
reveal_type(c, expected_text="int | Unknown")



# This sample tests the use of `Self` when used within a property
# or class property.

from typing_extensions import Self  # pyright: ignore[reportMissingModuleSource]


class A:
    @property
    def one(self) -> Self: ...

    @classmethod
    @property
    def two(cls) -> type[Self]: ...


class B(A): ...


reveal_type(A().one, expected_text="A")
reveal_type(A.two, expected_text="type[A]")

reveal_type(B().one, expected_text="B")
reveal_type(B.two, expected_text="type[B]")



# This sample tests the case where a type variable is bound to a union.

from typing import Callable, TypeVar

T = TypeVar("T")
IntStr = str | int
T1 = TypeVar("T1", bound=IntStr)
T2 = TypeVar("T2", bound=IntStr)


def custom_eq(x: IntStr, y: IntStr) -> bool:
    return True


def eq(f: Callable[[T1], T2], x: T1, y: T2) -> bool:
    return custom_eq(f(x), y)



# This sample tests that a ClassVar is disallowed when used in a
# NamedTuple or TypedDict class as reflected in the runtime.

from typing import ClassVar, NamedTuple, TypedDict


class NT1(NamedTuple):
    # This should generate an error.
    x: ClassVar

    # This should generate an error.
    y: ClassVar[int]


class TD1(TypedDict):
    # This should generate an error.
    x: ClassVar

    # This should generate an error.
    y: ClassVar[int]



# This sample tests the case where a protocol implementation uses a
# method-scoped type variable.

from typing import Protocol, Sequence, TypeVar

Input = TypeVar("Input", contravariant=True)
Output = TypeVar("Output", covariant=True)
T = TypeVar("T")


class ProtoA(Protocol[Input, Output]):
    def __call__(self, input: Input) -> Output:
        raise NotImplementedError


class ImplA:
    def __call__(self, input: Sequence[T]) -> T:
        return input[0]


v1: ProtoA[Sequence[int], int] = ImplA()



# This sample verifies the case where a property returns a callable.

from typing import Callable


class ClassA:
    def __init__(self, converter: Callable[[str, int], int]) -> None:
        self.converter = converter

    @property
    def converter_prop(self) -> Callable[[str, int], int]:
        return self.converter


def str_to_int(arg: str, base: int) -> int:
    return int(arg, base=base)


obj = ClassA(str_to_int)
val1: int = obj.converter("123", 10)
val2: int = obj.converter_prop("123", 10)

reveal_type(obj.converter, expected_text="(str, int) -> int")
reveal_type(obj.converter_prop, expected_text="(str, int) -> int")



# This sample tests a member access when the member is a class
# that inherits from Any.

from unittest.mock import Mock


class MockProducer:
    produce: type[Mock] = Mock


reveal_type(MockProducer.produce, expected_text="type[Mock]")
reveal_type(MockProducer().produce, expected_text="type[Mock]")


reveal_type(MockProducer.produce(), expected_text="Mock")
reveal_type(MockProducer().produce(), expected_text="Mock")



# This sample tests the constraint solver's special-case handling of
# Optional[T] within a function.

from typing import TypeVar

_T = TypeVar("_T")


def func1(v: _T | None) -> _T:
    if v is None:
        raise ValueError
    return v


def func2(v: _T) -> _T:
    if v is None:
        raise ValueError
    return v


f: int | None = None

a: int = func1(f)

# This should generate an error because type var _T
# should be matched to "Optional[int]".
b: int = func2(f)



# This sample tests a few cases that involve a loop with a conditional. These
# case regressed and were not caught by any other test cases.

from typing import TypeVar, Any

T = TypeVar("T")


def func1(x: T) -> T: ...


def func2(schema: bool): ...


def func3(v1: list[bool], v2: int | str):
    for _ in v1:
        if v2 in set([1, 2, 3]):
            func1(v2)

            # This should generate an error.
            func2(v2)


def func4(x: Any, b: Any):
    a = x
    while a < 1:
        if a:
            b = int(b)
        else:
            b = a

        if b:
            # This should generate an error.
            return a.x(dummy)



# This sample tests the type checker's handling of imaginary
# and complex numbers.

a = 3.2j

b = a + 4

c = 1.2 * a


def requires_complex(val: complex): ...


requires_complex(a)
requires_complex(b)
requires_complex(c)



# This sample tests support for inlined TypedDict definitions.

from typing import NotRequired, ReadOnly, Required, TypedDict


td1: TypedDict[{"a": int, "b": str}] = {"a": 0, "b": ""}

td2: TypedDict[{"a": TypedDict[{"b": int}]}] = {"a": {"b": 0}}

td3: TypedDict[{"a": "list[float]"}] = {"a": [3]}

td4: TypedDict[
    {"a": NotRequired[int], "b": Required[int], "c": NotRequired[ReadOnly[int]]}
] = {"b": 3}

# This should generate an error because dictionary comprehensions
# are not allowed.
err1: TypedDict[{"a": int for _ in range(1)}]

# This should generate an error because unpacked dictionary
# entries are not allowed.
err2: TypedDict[{**{"a": int}}]

# This should generate an error because an extra type argument is provided.
err3: TypedDict[{"a": int}, str]

# This should generate an error because TypedDict cannot be used without
# a subscript in this context.
err4: TypedDict

# This should generate an error because a dict expression is not a
# valid type expression by itself.
err5: TypedDict[{"a": {"b": int}}] = {"a": {"b": 0}}


def func1(val: TypedDict[{"a": int}]) -> TypedDict[{"a": int}]:
    return {"a": val["a"] + 1}


func1({"a": 3})


type TA1[T] = TypedDict[{"a": int, "b": T, "c": NotRequired[int]}]


class Outer1[T]:
    attr1: TypedDict[{"a": list[T]}]

    def __init__(self, v: T) -> None:
        self.attr1 = {"a": [v]}



# This sample checks the handling of callable types that are narrowed
# to a particular type using an isinstance type narrowing test.

from typing import Callable, ParamSpec, Protocol, Union, runtime_checkable

P = ParamSpec("P")


class ClassA:
    def __call__(self, arg: int, bar: str) -> None:
        raise NotImplementedError


@runtime_checkable
class ClassB(Protocol):
    def __call__(self, arg: int) -> None:
        raise NotImplementedError


@runtime_checkable
class ClassC(Protocol):
    def __call__(self, arg: str) -> None:
        raise NotImplementedError


def check_callable1(val: Union[Callable[[int, str], None], Callable[[int], None]]):
    if isinstance(val, ClassA):
        reveal_type(val, expected_text="ClassA")
    else:
        # This doesn't get narrowed because `ClassA` is not a runtime checkable protocol.
        reveal_type(val, expected_text="((int, str) -> None) | ((int) -> None)")


def check_callable2(val: Union[Callable[[int, str], None], Callable[[int], None]]):
    if isinstance(val, ClassB):
        reveal_type(val, expected_text="((int, str) -> None) | ((int) -> None)")
    else:
        reveal_type(val, expected_text="Never")


def check_callable3(val: Union[Callable[[int, str], None], Callable[[int], None]]):
    if isinstance(val, ClassC):
        reveal_type(val, expected_text="((int, str) -> None) | ((int) -> None)")
    else:
        reveal_type(val, expected_text="Never")


def check_callable4(val: Union[type, Callable[[int], None]]):
    if isinstance(val, type):
        reveal_type(val, expected_text="type")
    else:
        reveal_type(val, expected_text="(int) -> None")


def check_callable5(fn: Callable[P, None]) -> None:
    if isinstance(fn, ClassA):
        reveal_type(fn, expected_text="ClassA")
    else:
        reveal_type(fn, expected_text="(**P@check_callable5) -> None")


def check_callable6(o: object | Callable[[int], int]):
    if isinstance(o, Callable):
        reveal_type(o, expected_text="((...) -> Unknown) | ((int) -> int)")
    else:
        reveal_type(o, expected_text="object")



# This sample tests the reportUninitializedInstanceVariable when applied
# to a concrete implementation of an abstract base class that defines
# (but does not assign) variables.

from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import NamedTuple, final


class Abstract1(ABC):
    x: str


@final
# This should generate an error because x is unimplemented.
class A(Abstract1):
    pass


class B(Abstract1):
    pass


@final
class C(Abstract1):
    x = ""


@final
class D(Abstract1):
    def __init__(self):
        self.x = ""


class Abstract2(Abstract1):
    y: str


@final
# This should generate an error because x and y are unimplemented.
class E(Abstract2):
    pass


class Abstract3(Abstract1):
    x = ""


@final
class G(Abstract3):
    pass


class H(NamedTuple):
    x: int


@dataclass
class IAbstract(ABC):
    p1: str
    p2: int = field(init=False)


@final
@dataclass
# This should generate an error because p2 is uninitialized.
class I(IAbstract):
    p3: int



# This sample tests that a generic type alias can use
# a compatible bound TypeVar.

from typing import Generic, TypeVar

D = TypeVar("D", bool, int, float, object)
E = TypeVar("E", bool, int, float, object)


class Gen(Generic[D]):
    pass


GenAlias = Gen[E]



# This sample tests the case where an accessed member is a
# method that has a "self" or "cls" parameter with no explicit
# type annotation and an inferred type that is based on this value.


class A:
    async def get(self):
        return self


class B(A):
    pass


async def run():
    val1 = await A().get()
    reveal_type(val1, expected_text="A")

    val2 = await B().get()
    reveal_type(val2, expected_text="B")



# This sample tests basic support for PEP 696 -- default types for TypeVars.

from typing import Any
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    TypeVar,
    TypeVarTuple,
    ParamSpec,
    Unpack,
)

S1 = TypeVar("S1")
S2 = TypeVar("S2", bound=int)
S3 = TypeVar("S3", bytes, str)

Ts0 = TypeVarTuple("Ts0")

P0 = ParamSpec("P0")


T1 = TypeVar("T1", default=int)

# This should generate an error because default must be a type expression.
T2 = TypeVar("T2", default=3)

TInt = TypeVar("TInt", bound=int)
T3 = TypeVar("T3", bound=float, default=TInt)

# This should generate an error because default must be a subtype of bound.
T4 = TypeVar("T4", bound=int, default=float)

# This should generate an error because S1 is not a subtype of int.
T6 = TypeVar("T6", bound=int, default=S1)

T7 = TypeVar("T7", bound=float, default=S2)

# This should generate an error because S3 is not a subtype of int.
T8 = TypeVar("T8", bound=float, default=S3)

T9 = TypeVar("T9", bound=list[Any], default=list[S1])

T10 = TypeVar("T10", bytes, str, default=str)

# This should generate an error because str | bytes isn't one of the constrained types.
T11 = TypeVar("T11", bytes, str, default=str | bytes)

# This should generate an error because S1 isn't one of the constrained types.
T12 = TypeVar("T12", bytes, str, default=S1)

T13 = TypeVar("T13", int, str)
T14 = TypeVar("T14", int, str, bool, default=T13)

# This should generate an error because the constraints for T13 are not compatible.
T15 = TypeVar("T15", int, complex, bool, default=T13)

T16 = TypeVar("T16", bound=int)
T17 = TypeVar("T17", int, complex, bool, default=T16)

# This should generate an error because the type of T16 is not compatible.
T18 = TypeVar("T18", str, list, default=T16)


Ts1 = TypeVarTuple("Ts1", default=Unpack[tuple[int]])

# This should generate an error because default must be unpacked tuple.
Ts2 = TypeVarTuple("Ts2", default=tuple[int])

# This should generate an error because default must be unpacked tuple.
Ts3 = TypeVarTuple("Ts3", default=int)

Ts4 = TypeVarTuple("Ts4", default=Unpack[Ts0])

# This should generate an error because default must be unpacked.
Ts5 = TypeVarTuple("Ts5", default=Ts0)

Ts6 = TypeVarTuple("Ts6", default=Unpack[tuple[int, ...]])

Ts7 = TypeVarTuple("Ts7", default=Unpack[tuple[S1, S2]])


P1 = ParamSpec("P1", default=[])

P2 = ParamSpec("P2", default=[int, str, None, int | None])

P3 = ParamSpec("P3", default=[int, S1])

P4 = ParamSpec("P4", default=[int])

P5 = ParamSpec("P5", default=...)

# This should generate an error because ParamSpec must be a list of types.
P6 = ParamSpec("P6", default=int)

# This should generate an error because ParamSpec must be a list of types.
P7 = ParamSpec("P7", default=3)

# This should generate an error because ParamSpec must be a list of types.
P8 = ParamSpec("P8", default=(1, int))

P9 = ParamSpec("P9", default=P0)



# This sample tests the type analyzer's type narrowing logic for
# conditions of the form "X is ...", "X is not ...",
# "X == .." and "X != ...".

import types
from typing import Any, TypeVar

_T = TypeVar("_T", str, types.EllipsisType)


def func1(val: int | ellipsis):
    if val is not ...:
        reveal_type(val, expected_text="int")
    else:
        reveal_type(val, expected_text="EllipsisType")


def func2(val: _T):
    if val is ...:
        reveal_type(val, expected_text="EllipsisType*")
    else:
        reveal_type(val, expected_text="str*")


def func3(val: int | types.EllipsisType):
    if val != ...:
        reveal_type(val, expected_text="int")
    else:
        reveal_type(val, expected_text="EllipsisType")


def func4(val: int | ellipsis):
    if not val == ...:
        reveal_type(val, expected_text="int")
    else:
        reveal_type(val, expected_text="EllipsisType")


def func5(val: object):
    if val is ...:
        reveal_type(val, expected_text="EllipsisType")
    else:
        reveal_type(val, expected_text="object")


def func6(val: Any | types.EllipsisType):
    if val is not ...:
        reveal_type(val, expected_text="Any")
    else:
        reveal_type(val, expected_text="EllipsisType")



# This sample tests reporting of deprecated magic methods.

# pyright: reportMissingModuleSource=false

from typing import Self, overload

from typing_extensions import deprecated


class ClassA:
    @deprecated("Adding a str is deprecated")
    @overload
    def __add__(self, other: str) -> Self: ...

    @overload
    def __add__(self, other: object) -> Self: ...

    def __add__(self, other: object) -> Self: ...

    @deprecated("Bool is not supported")
    def __bool__(self) -> bool: ...

    @deprecated("Negation is no longer supported")
    def __neg__(self) -> Self: ...


a = ClassA()

v1 = a + 1

# This should be marked as deprecated.
v2 = a + ""

a += 1

# This should be marked as deprecated.
a += ""


# This should be marked as deprecated.
v3 = -a

# This should be marked as deprecated.
v4 = not a
